@@ -24,6 +24,7 @@
 # SOFTWARE.
 import asyncio
 import ast
+import errno
 import os
 import time
 import traceback
@@ -41,18 +42,19 @@ from aiorpcx import timeout_after, TaskTimeout, ignore_after
 
 from . import util
 from .network import Network
-from .util import (json_decode, to_bytes, to_string, profiler, standardize_path, constant_time_compare)
+from .util import (json_decode, to_bytes, to_string, profiler, standardize_path, constant_time_compare, InvalidPassword)
 from .invoices import PR_PAID, PR_EXPIRED
 from .util import log_exceptions, ignore_exceptions, randrange, OldTaskGroup
 from .util import EventListener, event_listener
 from .wallet import Wallet, Abstract_Wallet
 from .storage import WalletStorage
-from .wallet_db import WalletDB
+from .wallet_db import WalletDB, WalletRequiresSplit, WalletRequiresUpgrade, WalletUnfinished
 from .commands import known_commands, Commands
 from .simple_config import SimpleConfig
 from .exchange_rate import FxThread
 from .logging import get_logger, Logger
 from . import GuiImportError
+from .plugin import run_hook, Plugins
 
 if TYPE_CHECKING:
     from electrum import gui
@@ -68,7 +70,7 @@ def get_rpcsock_defaultpath(config: SimpleConfig):
     return os.path.join(config.path, 'daemon_rpc_socket')
 
 def get_rpcsock_default_type(config: SimpleConfig):
-    if config.get('rpcport'):
+    if config.RPC_PORT:
         return 'tcp'
     # Use unix domain sockets when available,
     # with the extra paranoia that in case windows "implements" them,
@@ -105,10 +107,11 @@ def get_file_descriptor(config: SimpleConfig):
 
 
 
-def request(config: SimpleConfig, endpoint, args=(), timeout=60):
+def request(config: SimpleConfig, endpoint, args=(), timeout: Union[float, int] = 60):
     lockfile = get_lockfile(config)
     while True:
         create_time = None
+        path = None
         try:
             with open(lockfile) as f:
                 socktype, address, create_time = ast.literal_eval(f.read())
@@ -126,7 +129,9 @@ def request(config: SimpleConfig, endpoint, args=(), timeout=60):
         server_url = 'http://%s:%d' % (host, port)
         auth = aiohttp.BasicAuth(login=rpc_user, password=rpc_password)
         loop = util.get_asyncio_loop()
-        async def request_coroutine():
+        async def request_coroutine(
+            *, socktype=socktype, path=path, auth=auth, server_url=server_url, endpoint=endpoint,
+        ):
             if socktype == 'unix':
                 connector = aiohttp.UnixConnector(path=path)
             elif socktype == 'tcp':
@@ -147,13 +152,22 @@ def request(config: SimpleConfig, endpoint, args=(), timeout=60):
         time.sleep(1.0)
 
 
+def wait_until_daemon_becomes_ready(*, config: SimpleConfig, timeout=5) -> bool:
+    t0 = time.monotonic()
+    while True:
+        if time.monotonic() > t0 + timeout:
+            return False  # timeout
+        try:
+            request(config, 'ping')
+            return True  # success
+        except DaemonNotRunning:
+            time.sleep(0.05)
+            continue
+
+
 def get_rpc_credentials(config: SimpleConfig) -> Tuple[str, str]:
-    rpc_user = config.get('rpcuser', None)
-    rpc_password = config.get('rpcpassword', None)
-    if rpc_user == '':
-        rpc_user = None
-    if rpc_password == '':
-        rpc_password = None
+    rpc_user = config.RPC_USERNAME or None
+    rpc_password = config.RPC_PASSWORD or None
     if rpc_user is None or rpc_password is None:
         rpc_user = 'user'
         bits = 128
@@ -162,8 +176,8 @@ def get_rpc_credentials(config: SimpleConfig) -> Tuple[str, str]:
         pw_b64 = b64encode(
             pw_int.to_bytes(nbytes, 'big'), b'-_')
         rpc_password = to_string(pw_b64, 'ascii')
-        config.set_key('rpcuser', rpc_user)
-        config.set_key('rpcpassword', rpc_password, save=True)
+        config.RPC_USERNAME = rpc_user
+        config.RPC_PASSWORD = rpc_password
     return rpc_user, rpc_password
 
 
@@ -248,17 +262,17 @@ class AuthenticatedServer(Logger):
 
 class CommandsServer(AuthenticatedServer):
 
-    def __init__(self, daemon, fd):
+    def __init__(self, daemon: 'Daemon', fd):
         rpc_user, rpc_password = get_rpc_credentials(daemon.config)
         AuthenticatedServer.__init__(self, rpc_user, rpc_password)
         self.daemon = daemon
         self.fd = fd
         self.config = daemon.config
-        sockettype = self.config.get('rpcsock', 'auto')
+        sockettype = self.config.RPC_SOCKET_TYPE
         self.socktype = sockettype if sockettype != 'auto' else get_rpcsock_default_type(self.config)
-        self.sockpath = self.config.get('rpcsockpath', get_rpcsock_defaultpath(self.config))
-        self.host = self.config.get('rpchost', '127.0.0.1')
-        self.port = self.config.get('rpcport', 0)
+        self.sockpath = self.config.RPC_SOCKET_FILEPATH or get_rpcsock_defaultpath(self.config)
+        self.host = self.config.RPC_HOST
+        self.port = self.config.RPC_PORT
         self.app = web.Application()
         self.app.router.add_post("/", self.handle)
         self.register_method(self.ping)
@@ -268,6 +282,14 @@ class CommandsServer(AuthenticatedServer):
             self.register_method(getattr(self.cmd_runner, cmdname))
         self.register_method(self.run_cmdline)
 
+    def _socket_config_str(self) -> str:
+        if self.socktype == 'unix':
+            return f"<socket type={self.socktype}, path={self.sockpath}>"
+        elif self.socktype == 'tcp':
+            return f"<socket type={self.socktype}, host={self.host}, port={self.port}>"
+        else:
+            raise Exception(f"unknown socktype '{self.socktype!r}'")
+
     async def run(self):
         self.runner = web.AppRunner(self.app)
         await self.runner.setup()
@@ -277,7 +299,10 @@ class CommandsServer(AuthenticatedServer):
             site = web.TCPSite(self.runner, self.host, self.port)
         else:
             raise Exception(f"unknown socktype '{self.socktype!r}'")
-        await site.start()
+        try:
+            await site.start()
+        except Exception as e:
+            raise Exception(f"failed to start CommandsServer at {self._socket_config_str()}. got exc: {e!r}") from None
         socket = site._server.sockets[0]
         if self.socktype == 'unix':
             addr = self.sockpath
@@ -293,9 +318,12 @@ class CommandsServer(AuthenticatedServer):
         return True
 
     async def gui(self, config_options):
+        # note: "config_options" is coming from the short-lived CLI-invocation,
+        #        while self.config is the config of the long-lived daemon process.
+        #       "config_options" should have priority.
         if self.daemon.gui_object:
             if hasattr(self.daemon.gui_object, 'new_window'):
-                path = self.config.get_wallet_path(use_gui_last_wallet=True)
+                path = config_options.get('wallet_path') or self.config.get_wallet_path(use_gui_last_wallet=True)
                 self.daemon.gui_object.new_window(path, config_options.get('url'))
                 response = "ok"
             else:
@@ -330,12 +358,12 @@ class CommandsServer(AuthenticatedServer):
 
 class WatchTowerServer(AuthenticatedServer):
 
-    def __init__(self, network, netaddress):
-        self.addr = netaddress
+    def __init__(self, network: 'Network', port:int):
+        self.port = port
         self.config = network.config
         self.network = network
-        watchtower_user = self.config.get('watchtower_user', '')
-        watchtower_password = self.config.get('watchtower_password', '')
+        watchtower_user = self.config.WATCHTOWER_SERVER_USER or ""
+        watchtower_password = self.config.WATCHTOWER_SERVER_PASSWORD or ""
         AuthenticatedServer.__init__(self, watchtower_user, watchtower_password)
         self.lnwatcher = network.local_watchtower
         self.app = web.Application()
@@ -346,9 +374,9 @@ class WatchTowerServer(AuthenticatedServer):
     async def run(self):
         self.runner = web.AppRunner(self.app)
         await self.runner.setup()
-        site = web.TCPSite(self.runner, host=str(self.addr.host), port=self.addr.port, ssl_context=self.config.get_ssl_context())
+        site = web.TCPSite(self.runner, host='localhost', port=self.port)
         await site.start()
-        self.logger.info(f"now running and listening. addr={self.addr}")
+        self.logger.info(f"running and listening on port {self.port}")
 
     async def get_ctn(self, *args):
         return await self.lnwatcher.get_ctn(*args)
@@ -357,109 +385,23 @@ class WatchTowerServer(AuthenticatedServer):
         return await self.lnwatcher.sweepstore.add_sweep_tx(*args)
 
 
-class PayServer(Logger, EventListener):
-
-    def __init__(self, daemon: 'Daemon', netaddress):
-        Logger.__init__(self)
-        self.addr = netaddress
-        self.daemon = daemon
-        self.config = daemon.config
-        self.pending = defaultdict(asyncio.Event)
-        self.register_callbacks()
-
-    @property
-    def wallet(self):
-        # FIXME specify wallet somehow?
-        return list(self.daemon.get_wallets().values())[0]
-
-    @event_listener
-    async def on_event_request_status(self, wallet, key, status):
-        if status == PR_PAID:
-            self.pending[key].set()
-
-    @ignore_exceptions
-    @log_exceptions
-    async def run(self):
-        root = self.config.get('payserver_root', '/r')
-        app = web.Application()
-        app.add_routes([web.get('/api/get_invoice', self.get_request)])
-        app.add_routes([web.get('/api/get_status', self.get_status)])
-        app.add_routes([web.get('/bip70/{key}.bip70', self.get_bip70_request)])
-        app.add_routes([web.static(root, os.path.join(os.path.dirname(__file__), 'www'))])
-        if self.config.get('payserver_allow_create_invoice'):
-            app.add_routes([web.post('/api/create_invoice', self.create_request)])
-        runner = web.AppRunner(app)
-        await runner.setup()
-        site = web.TCPSite(runner, host=str(self.addr.host), port=self.addr.port, ssl_context=self.config.get_ssl_context())
-        await site.start()
-        self.logger.info(f"now running and listening. addr={self.addr}")
-
-    async def create_request(self, request):
-        raise NotImplementedError()  # FIXME code here is broken
-        params = await request.post()
-        wallet = self.wallet
-        if 'amount_sat' not in params or not params['amount_sat'].isdigit():
-            raise web.HTTPUnsupportedMediaType()
-        amount = int(params['amount_sat'])
-        message = params['message'] or "donation"
-        payment_hash = wallet.lnworker.add_request(
-            amount_sat=amount,
-            message=message,
-            expiry=3600)
-        key = payment_hash.hex()
-        raise web.HTTPFound(self.root + '/pay?id=' + key)
-
-    async def get_request(self, r):
-        key = r.query_string
-        request = self.wallet.get_formatted_request(key)
-        return web.json_response(request)
-
-    async def get_bip70_request(self, r):
-        from .paymentrequest import make_request
-        key = r.match_info['key']
-        request = self.wallet.get_request(key)
-        if not request:
-            return web.HTTPNotFound()
-        pr = make_request(self.config, request)
-        return web.Response(body=pr.SerializeToString(), content_type='application/bitcoin-paymentrequest')
-
-    async def get_status(self, request):
-        ws = web.WebSocketResponse()
-        await ws.prepare(request)
-        key = request.query_string
-        info = self.wallet.get_formatted_request(key)
-        if not info:
-            await ws.send_str('unknown invoice')
-            await ws.close()
-            return ws
-        if info.get('status') == PR_PAID:
-            await ws.send_str(f'paid')
-            await ws.close()
-            return ws
-        if info.get('status') == PR_EXPIRED:
-            await ws.send_str(f'expired')
-            await ws.close()
-            return ws
-        while True:
-            try:
-                await asyncio.wait_for(self.pending[key].wait(), 1)
-                break
-            except asyncio.TimeoutError:
-                # send data on the websocket, to keep it alive
-                await ws.send_str('waiting')
-        await ws.send_str('paid')
-        await ws.close()
-        return ws
-
 
 
 class Daemon(Logger):
 
-    network: Optional[Network]
-    gui_object: Optional['gui.BaseElectrumGui']
+    network: Optional[Network] = None
+    gui_object: Optional['gui.BaseElectrumGui'] = None
+    watchtower: Optional['WatchTowerServer'] = None
 
     @profiler
-    def __init__(self, config: SimpleConfig, fd=None, *, listen_jsonrpc=True):
+    def __init__(
+        self,
+        config: SimpleConfig,
+        fd=None,
+        *,
+        listen_jsonrpc: bool = True,
+        start_network: bool = True,  # setting to False allows customising network settings before starting it
+    ):
         Logger.__init__(self)
         self.config = config
         self.listen_jsonrpc = listen_jsonrpc
@@ -470,53 +412,34 @@ class Daemon(Logger):
         if 'wallet_path' in config.cmdline_options:
             self.logger.warning("Ignoring parameter 'wallet_path' for daemon. "
                                 "Use the load_wallet command instead.")
+        self._plugins = None  # type: Optional[Plugins]
         self.asyncio_loop = util.get_asyncio_loop()
-        self.network = None
-        if not config.get('offline'):
+        if not self.config.NETWORK_OFFLINE:
             self.network = Network(config, daemon=self)
-        self.fx = FxThread(config, self.network)
-        self.gui_object = None
-        # path -> wallet;   make sure path is standardized.
+        self.fx = FxThread(config=config)
+        # wallet_key -> wallet
         self._wallets = {}  # type: Dict[str, Abstract_Wallet]
         self._wallet_lock = threading.RLock()
-        daemon_jobs = []
-        # Setup commands server
-        self.commands_server = None
-        if listen_jsonrpc:
-            self.commands_server = CommandsServer(self, fd)
-            daemon_jobs.append(self.commands_server.run())
-        # pay server
-        self.pay_server = None
-        payserver_address = self.config.get_netaddress('payserver_address')
-        if not config.get('offline') and payserver_address:
-            self.pay_server = PayServer(self, payserver_address)
-            daemon_jobs.append(self.pay_server.run())
-        # server-side watchtower
-        self.watchtower = None
-        watchtower_address = self.config.get_netaddress('watchtower_address')
-        if not config.get('offline') and watchtower_address:
-            self.watchtower = WatchTowerServer(self.network, watchtower_address)
-            daemon_jobs.append(self.watchtower.run)
-        if self.network:
-            self.network.start(jobs=[self.fx.run])
-            # prepare lightning functionality, also load channel db early
-            if self.config.get('use_gossip', False):
-                self.network.start_gossip()
 
         self._stop_entered = False
         self._stopping_soon_or_errored = threading.Event()
         self._stopped_event = threading.Event()
+
         self.taskgroup = OldTaskGroup()
-        asyncio.run_coroutine_threadsafe(self._run(jobs=daemon_jobs), self.asyncio_loop)
+        asyncio.run_coroutine_threadsafe(self._run(), self.asyncio_loop)
+        if start_network and self.network:
+            self.start_network()
+        # Setup commands server
+        self.commands_server = None
+        if listen_jsonrpc:
+            self.commands_server = CommandsServer(self, fd)
+            asyncio.run_coroutine_threadsafe(self.taskgroup.spawn(self.commands_server.run()), self.asyncio_loop)
 
     @log_exceptions
-    async def _run(self, jobs: Iterable = None):
-        if jobs is None:
-            jobs = []
+    async def _run(self):
         self.logger.info("starting taskgroup.")
         try:
             async with self.taskgroup as group:
-                [await group.spawn(job) for job in jobs]
                 await group.spawn(asyncio.Event().wait)  # run forever (until cancel)
         except Exception as e:
             self.logger.exception("taskgroup died.")
@@ -527,6 +450,31 @@ class Daemon(Logger):
             #       not see the exception (especially if the GUI did not start yet).
             self._stopping_soon_or_errored.set()
 
+    def start_network(self):
+        self.logger.info(f"starting network.")
+        assert not self.config.NETWORK_OFFLINE
+        assert self.network
+        # server-side watchtower
+        if watchtower_port := self.config.WATCHTOWER_SERVER_PORT:
+            self.watchtower = WatchTowerServer(self.network, watchtower_port)
+            asyncio.run_coroutine_threadsafe(self.taskgroup.spawn(self.watchtower.run), self.asyncio_loop)
+
+        self.network.start(jobs=[self.fx.run])
+        # prepare lightning functionality, also load channel db early
+        if self.config.LIGHTNING_USE_GOSSIP:
+            self.network.start_gossip()
+
+    @staticmethod
+    def _wallet_key_from_path(path) -> str:
+        """This does stricter path standardization than 'standardize_path'.
+        It is used for keying the _wallets dict, but not for the actual filesystem operations. (see #8495)
+        """
+        path = standardize_path(path)
+        # also resolve symlinks and windows network mounts/etc:
+        path = os.path.realpath(path)
+        path = os.path.normcase(path)
+        return str(path)
+
     def with_wallet_lock(func):
         def func_wrapper(self: 'Daemon', *args, **kwargs):
             with self._wallet_lock:
@@ -534,55 +482,51 @@ class Daemon(Logger):
         return func_wrapper
 
     @with_wallet_lock
-    def load_wallet(self, path, password, *, manual_upgrades=True) -> Optional[Abstract_Wallet]:
+    def load_wallet(self, path, password, *, upgrade=False) -> Optional[Abstract_Wallet]:
         path = standardize_path(path)
+        wallet_key = self._wallet_key_from_path(path)
         # wizard will be launched if we return
-        if path in self._wallets:
-            wallet = self._wallets[path]
+        if wallet := self._wallets.get(wallet_key):
             return wallet
-        wallet = self._load_wallet(path, password, manual_upgrades=manual_upgrades, config=self.config)
-        if wallet is None:
-            return
+        wallet = self._load_wallet(path, password, upgrade=upgrade, config=self.config)
         wallet.start_network(self.network)
-        self._wallets[path] = wallet
+        self.add_wallet(wallet)
         return wallet
 
     @staticmethod
+    @profiler
     def _load_wallet(
             path,
             password,
             *,
-            manual_upgrades: bool = True,
+            upgrade: bool = False,
             config: SimpleConfig,
     ) -> Optional[Abstract_Wallet]:
         path = standardize_path(path)
         storage = WalletStorage(path)
         if not storage.file_exists():
-            return
+            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), path)
         if storage.is_encrypted():
             if not password:
-                return
+                raise InvalidPassword('No password given')
             storage.decrypt(password)
         # read data, pass it to db
-        db = WalletDB(storage.read(), manual_upgrades=manual_upgrades)
-        if db.requires_split():
-            return
-        if db.requires_upgrade():
-            return
+        db = WalletDB(storage.read(), storage=storage, upgrade=upgrade)
         if db.get_action():
-            return
-        wallet = Wallet(db, storage, config=config)
+            raise WalletUnfinished(db)
+        wallet = Wallet(db, config=config)
         return wallet
 
     @with_wallet_lock
     def add_wallet(self, wallet: Abstract_Wallet) -> None:
         path = wallet.storage.path
-        path = standardize_path(path)
-        self._wallets[path] = wallet
+        wallet_key = self._wallet_key_from_path(path)
+        self._wallets[wallet_key] = wallet
+        run_hook('daemon_wallet_loaded', self, wallet)
 
     def get_wallet(self, path: str) -> Optional[Abstract_Wallet]:
-        path = standardize_path(path)
-        return self._wallets.get(path)
+        wallet_key = self._wallet_key_from_path(path)
+        return self._wallets.get(wallet_key)
 
     @with_wallet_lock
     def get_wallets(self) -> Dict[str, Abstract_Wallet]:
@@ -604,18 +548,26 @@ class Daemon(Logger):
     @with_wallet_lock
     async def _stop_wallet(self, path: str) -> bool:
         """Returns True iff a wallet was found."""
-        path = standardize_path(path)
-        wallet = self._wallets.pop(path, None)
+        wallet_key = self._wallet_key_from_path(path)
+        wallet = self._wallets.pop(wallet_key, None)
         if not wallet:
             return False
         await wallet.stop()
         return True
 
     def run_daemon(self):
+        # init plugins
+        self._plugins = Plugins(self.config, 'cmdline')
+        # block until we are stopping
         try:
             self._stopping_soon_or_errored.wait()
         except KeyboardInterrupt:
-            asyncio.run_coroutine_threadsafe(self.stop(), self.asyncio_loop).result()
+            self.logger.info("got KeyboardInterrupt")
+        # we either initiate shutdown now,
+        # or it has already been initiated (in which case this is a no-op):
+        self.logger.info("run_daemon is calling stop()")
+        asyncio.run_coroutine_threadsafe(self.stop(), self.asyncio_loop).result()
+        # wait until "stop" finishes:
         self._stopped_event.wait()
 
     async def stop(self):
@@ -637,6 +589,11 @@ class Daemon(Logger):
                     if self.network:
                         await group.spawn(self.network.stop(full_shutdown=True))
                     await group.spawn(self.taskgroup.cancel_remaining())
+            if self._plugins:
+                self.logger.info("stopping plugins")
+                self._plugins.stop()
+                async with ignore_after(1):
+                    await self._plugins.stopped_event_async.wait()
         finally:
             if self.listen_jsonrpc:
                 self.logger.info("removing lockfile")
@@ -644,18 +601,20 @@ class Daemon(Logger):
             self.logger.info("stopped")
             self._stopped_event.set()
 
-    def run_gui(self, config, plugins):
+    def run_gui(self) -> None:
+        assert self.config
         threading.current_thread().name = 'GUI'
-        gui_name = config.get('gui', 'qt')
+        gui_name = self.config.GUI_NAME
         if gui_name in ['lite', 'classic']:
             gui_name = 'qt'
+        self._plugins = Plugins(self.config, gui_name)  # init plugins
         self.logger.info(f'launching GUI: {gui_name}')
         try:
             try:
                 gui = __import__('electrum.gui.' + gui_name, fromlist=['electrum'])
             except GuiImportError as e:
                 sys.exit(str(e))
-            self.gui_object = gui.ElectrumGui(config=config, daemon=self, plugins=plugins)
+            self.gui_object = gui.ElectrumGui(config=self.config, daemon=self, plugins=self._plugins)
             if not self._stop_entered:
                 self.gui_object.main()
             else:
@@ -682,27 +641,36 @@ class Daemon(Logger):
             if not os.path.isfile(path):
                 continue
             wallet = self.get_wallet(path)
+            # note: we only create a new wallet object if one was not loaded into the wallet already.
+            #       This is to avoid having two wallet objects contending for the same file.
+            #       Take care: this only works if the daemon knows about all wallet objects.
+            #                  if other code already has created a Wallet() for a file but did not tell the daemon,
+            #                  hard-to-understand bugs will follow...
             if wallet is None:
                 try:
-                    wallet = self._load_wallet(path, old_password, manual_upgrades=False, config=self.config)
+                    wallet = self._load_wallet(path, old_password, upgrade=True, config=self.config)
                 except util.InvalidPassword:
                     pass
                 except Exception:
                     self.logger.exception(f'failed to load wallet at {path!r}:')
-                    pass
             if wallet is None:
                 failed.append(path)
                 continue
             if not wallet.storage.is_encrypted():
                 is_unified = False
             try:
-                wallet.check_password(old_password)
+                try:
+                    wallet.check_password(old_password)
+                    old_password_real = old_password
+                except util.InvalidPassword:
+                    wallet.check_password(None)
+                    old_password_real = None
             except Exception:
                 failed.append(path)
                 continue
             if new_password:
                 self.logger.info(f'updating password for wallet: {path!r}')
-                wallet.update_password(old_password, new_password, encrypt_storage=True)
+                wallet.update_password(old_password_real, new_password, encrypt_storage=True)
         can_be_unified = failed == []
         is_unified = can_be_unified and is_unified
         return can_be_unified, is_unified
