@@ -5,27 +5,27 @@ import os
 import sys
 import html
 import threading
-import asyncio
 from typing import TYPE_CHECKING, Set
 
-from PyQt5.QtCore import (pyqtSlot, pyqtSignal, pyqtProperty, QObject, QUrl, QLocale,
+from PyQt6.QtCore import (pyqtSlot, pyqtSignal, pyqtProperty, QObject, QT_VERSION_STR, PYQT_VERSION_STR,
                           qInstallMessageHandler, QTimer, QSortFilterProxyModel)
-from PyQt5.QtGui import QGuiApplication, QFontDatabase
-from PyQt5.QtQml import qmlRegisterType, qmlRegisterUncreatableType, QQmlApplicationEngine
+from PyQt6.QtGui import QGuiApplication, QFontDatabase, QScreen
+from PyQt6.QtQml import qmlRegisterType, qmlRegisterUncreatableType, QQmlApplicationEngine
 
 from electrum import version, constants
 from electrum.i18n import _
 from electrum.logging import Logger, get_logger
-from electrum.util import BITCOIN_BIP21_URI_SCHEME, LIGHTNING_URI_SCHEME
+from electrum.bip21 import BITCOIN_BIP21_URI_SCHEME, LIGHTNING_URI_SCHEME
 from electrum.base_crash_reporter import BaseCrashReporter, EarlyExceptionsQueue
 from electrum.network import Network
+from electrum.plugin import run_hook
 
 from .qeconfig import QEConfig
 from .qedaemon import QEDaemon
 from .qenetwork import QENetwork
 from .qewallet import QEWallet
 from .qeqr import QEQRParser, QEQRImageProvider, QEQRImageProviderHelper
-from .qewalletdb import QEWalletDB
+from .qeqrscanner import QEQRScanner
 from .qebitcoin import QEBitcoin
 from .qefx import QEFX
 from .qetxfinalizer import QETxFinalizer, QETxRbfFeeBumper, QETxCpfpFeeBumper, QETxCanceller
@@ -60,6 +60,7 @@ if 'ANDROID_DATA' in os.environ:
 
 notification = None
 
+
 class QEAppController(BaseCrashReporter, QObject):
     _dummy = pyqtSignal()
     userNotify = pyqtSignal(str, str)
@@ -69,11 +70,13 @@ class QEAppController(BaseCrashReporter, QObject):
     sendingBugreportSuccess = pyqtSignal(str)
     sendingBugreportFailure = pyqtSignal(str)
     secureWindowChanged = pyqtSignal()
+    wantCloseChanged = pyqtSignal()
 
-    def __init__(self, qedaemon: 'QEDaemon', plugins: 'Plugins'):
+    def __init__(self, qeapp: 'ElectrumQmlApplication', qedaemon: 'QEDaemon', plugins: 'Plugins'):
         BaseCrashReporter.__init__(self, None, None, None)
         QObject.__init__(self)
 
+        self._app = qeapp
         self._qedaemon = qedaemon
         self._plugins = plugins
         self.config = qedaemon.daemon.config
@@ -99,6 +102,8 @@ class QEAppController(BaseCrashReporter, QObject):
         if self.isAndroid():
             self.bindIntent()
 
+        self._want_close = False
+
     def on_wallet_loaded(self):
         qewallet = self._qedaemon.currentWallet
         if not qewallet:
@@ -178,6 +183,16 @@ class QEAppController(BaseCrashReporter, QObject):
         if self._intent:
             self.on_new_intent(self._intent)
 
+    @pyqtProperty(bool, notify=wantCloseChanged)
+    def wantClose(self):
+        return self._want_close
+
+    @wantClose.setter
+    def wantClose(self, want_close):
+        if want_close != self._want_close:
+            self._want_close = want_close
+            self.wantCloseChanged.emit()
+
     @pyqtSlot(str, str)
     def doShare(self, data, title):
         if not self.isAndroid():
@@ -224,12 +239,18 @@ class QEAppController(BaseCrashReporter, QObject):
         return s
 
     @pyqtSlot(str, bool)
-    def setPluginEnabled(self, plugin, enabled):
+    def setPluginEnabled(self, plugin: str, enabled: bool):
         if enabled:
             self._plugins.enable(plugin)
+            # note: all enabled plugins will receive this hook:
+            run_hook('init_qml', self._app)
         else:
             self._plugins.disable(plugin)
 
+    @pyqtSlot(str, result=bool)
+    def isPluginEnabled(self, plugin: str):
+        return bool(self._plugins.get(plugin))
+
     @pyqtSlot(result=bool)
     def isAndroid(self):
         return 'ANDROID_DATA' in os.environ
@@ -272,7 +293,7 @@ class QEAppController(BaseCrashReporter, QObject):
 
     @pyqtSlot()
     def showNever(self):
-        self.config.set_key(BaseCrashReporter.config_key, False)
+        self.config.SHOW_CRASH_REPORTER = False
 
     @pyqtSlot(str)
     def setCrashUserText(self, text):
@@ -306,11 +327,14 @@ class QEAppController(BaseCrashReporter, QObject):
     def secureWindow(self, secure):
         if not self.isAndroid():
             return
+        if self.config.GUI_QML_ALWAYS_ALLOW_SCREENSHOTS:
+            return
         if self._secureWindow != secure:
             jpythonActivity.setSecureWindow(secure)
             self._secureWindow = secure
             self.secureWindowChanged.emit()
 
+
 class ElectrumQmlApplication(QGuiApplication):
 
     _valid = True
@@ -322,10 +346,17 @@ class ElectrumQmlApplication(QGuiApplication):
 
         ElectrumQmlApplication._daemon = daemon
 
+        # TODO QT6 order of declaration is important now?
+        qmlRegisterType(QEAmount, 'org.electrum', 1, 0, 'Amount')
+        qmlRegisterType(QENewWalletWizard, 'org.electrum', 1, 0, 'QNewWalletWizard')
+        qmlRegisterType(QEServerConnectWizard, 'org.electrum', 1, 0, 'QServerConnectWizard')
+        qmlRegisterType(QEFilterProxyModel, 'org.electrum', 1, 0, 'FilterProxyModel')
+        qmlRegisterType(QSortFilterProxyModel, 'org.electrum', 1, 0, 'QSortFilterProxyModel')
+
         qmlRegisterType(QEWallet, 'org.electrum', 1, 0, 'Wallet')
-        qmlRegisterType(QEWalletDB, 'org.electrum', 1, 0, 'WalletDB')
         qmlRegisterType(QEBitcoin, 'org.electrum', 1, 0, 'Bitcoin')
         qmlRegisterType(QEQRParser, 'org.electrum', 1, 0, 'QRParser')
+        qmlRegisterType(QEQRScanner, 'org.electrum', 1, 0, 'QRScanner')
         qmlRegisterType(QEFX, 'org.electrum', 1, 0, 'FX')
         qmlRegisterType(QETxFinalizer, 'org.electrum', 1, 0, 'TxFinalizer')
         qmlRegisterType(QEInvoice, 'org.electrum', 1, 0, 'Invoice')
@@ -342,19 +373,21 @@ class ElectrumQmlApplication(QGuiApplication):
         qmlRegisterType(QETxCanceller, 'org.electrum', 1, 0, 'TxCanceller')
         qmlRegisterType(QEBip39RecoveryListModel, 'org.electrum', 1, 0, 'Bip39RecoveryListModel')
 
-        qmlRegisterUncreatableType(QEAmount, 'org.electrum', 1, 0, 'Amount', 'Amount can only be used as property')
-        qmlRegisterUncreatableType(QENewWalletWizard, 'org.electrum', 1, 0, 'QNewWalletWizard', 'QNewWalletWizard can only be used as property')
-        qmlRegisterUncreatableType(QEServerConnectWizard, 'org.electrum', 1, 0, 'QServerConnectWizard', 'QServerConnectWizard can only be used as property')
-        qmlRegisterUncreatableType(QEFilterProxyModel, 'org.electrum', 1, 0, 'FilterProxyModel', 'FilterProxyModel can only be used as property')
-        qmlRegisterUncreatableType(QSortFilterProxyModel, 'org.electrum', 1, 0, 'QSortFilterProxyModel', 'QSortFilterProxyModel can only be used as property')
+        # TODO QT6: these were declared as uncreatable, but that doesn't seem to work for pyqt6
+        # qmlRegisterUncreatableType(QEAmount, 'org.electrum', 1, 0, 'Amount', 'Amount can only be used as property')
+        # qmlRegisterUncreatableType(QENewWalletWizard, 'org.electrum', 1, 0, 'QNewWalletWizard', 'QNewWalletWizard can only be used as property')
+        # qmlRegisterUncreatableType(QEServerConnectWizard, 'org.electrum', 1, 0, 'QServerConnectWizard', 'QServerConnectWizard can only be used as property')
+        # qmlRegisterUncreatableType(QEFilterProxyModel, 'org.electrum', 1, 0, 'FilterProxyModel', 'FilterProxyModel can only be used as property')
+        # qmlRegisterUncreatableType(QSortFilterProxyModel, 'org.electrum', 1, 0, 'QSortFilterProxyModel', 'QSortFilterProxyModel can only be used as property')
 
         self.engine = QQmlApplicationEngine(parent=self)
 
         screensize = self.primaryScreen().size()
 
-        self.qr_ip = QEQRImageProvider((7/8)*min(screensize.width(), screensize.height()))
+        qr_size = min(screensize.width(), screensize.height()) * 7/8
+        self.qr_ip = QEQRImageProvider(qr_size)
         self.engine.addImageProvider('qrgen', self.qr_ip)
-        self.qr_ip_h = QEQRImageProviderHelper((7/8)*min(screensize.width(), screensize.height()))
+        self.qr_ip_h = QEQRImageProviderHelper(qr_size)
 
         # add a monospace font as we can't rely on device having one
         self.fixedFont = 'PT Mono'
@@ -368,8 +401,8 @@ class ElectrumQmlApplication(QGuiApplication):
         self.plugins = plugins
         self._qeconfig = QEConfig(config)
         self._qenetwork = QENetwork(daemon.network, self._qeconfig)
-        self.daemon = QEDaemon(daemon)
-        self.appController = QEAppController(self.daemon, self.plugins)
+        self.daemon = QEDaemon(daemon, self.plugins)
+        self.appController = QEAppController(self, self.daemon, self.plugins)
         self._maxAmount = QEAmount(is_max=True)
         self.context.setContextProperty('AppController', self.appController)
         self.context.setContextProperty('Config', self._qeconfig)
@@ -380,8 +413,9 @@ class ElectrumQmlApplication(QGuiApplication):
         self.context.setContextProperty('QRIP', self.qr_ip_h)
         self.context.setContextProperty('BUILD', {
             'electrum_version': version.ELECTRUM_VERSION,
-            'apk_version': version.APK_VERSION,
-            'protocol_version': version.PROTOCOL_VERSION
+            'protocol_version': version.PROTOCOL_VERSION,
+            'qt_version': QT_VERSION_STR,
+            'pyqt_version': PYQT_VERSION_STR
         })
 
         self.plugins.load_plugin('trustedcoin')
@@ -405,6 +439,7 @@ class ElectrumQmlApplication(QGuiApplication):
             return
         self.logger.warning(file)
 
+
 class Exception_Hook(QObject, Logger):
     _report_exception = pyqtSignal(object, object, object, object)
 
@@ -425,7 +460,7 @@ class Exception_Hook(QObject, Logger):
 
     @classmethod
     def maybe_setup(cls, *, config: 'SimpleConfig', wallet: 'Abstract_Wallet' = None, slot = None) -> None:
-        if not config.get(BaseCrashReporter.config_key, default=True):
+        if not config.SHOW_CRASH_REPORTER:
             EarlyExceptionsQueue.set_hook_as_ready()  # flush already queued exceptions
             return
         if not cls._INSTANCE:
