@@ -1,13 +1,15 @@
 import threading
 from concurrent.futures import CancelledError
 from asyncio.exceptions import TimeoutError
+from typing import TYPE_CHECKING, Optional
 
-from PyQt5.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject
+from PyQt6.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject
 
 from electrum.i18n import _
 from electrum.gui import messages
 from electrum.util import bfh
-from electrum.lnutil import extract_nodeid, LNPeerAddr, ln_dummy_address
+from electrum.lnutil import extract_nodeid, ConnStringFormatError
+from electrum.bitcoin import DummyAddress
 from electrum.lnworker import hardcoded_trampoline_nodes
 from electrum.logging import get_logger
 
@@ -19,25 +21,30 @@ from .qewallet import QEWallet
 
 
 class QEChannelOpener(QObject, AuthMixin):
-    def __init__(self, parent=None):
-        super().__init__(parent)
-
     _logger = get_logger(__name__)
 
-    _wallet = None
-    _nodeid = None
-    _amount = QEAmount()
-    _valid = False
-    _opentx = None
-    _txdetails = None
-
     validationError = pyqtSignal([str,str], arguments=['code','message'])
     conflictingBackup = pyqtSignal([str], arguments=['message'])
     channelOpening = pyqtSignal([str], arguments=['peer'])
     channelOpenError = pyqtSignal([str], arguments=['message'])
-    channelOpenSuccess = pyqtSignal([str,bool,int], arguments=['cid','has_onchain_backup','min_depth','tx_complete'])
+    channelOpenSuccess = pyqtSignal([str, bool, int, bool],
+                                    arguments=['cid', 'has_onchain_backup', 'min_depth', 'tx_complete'])
 
-    dataChanged = pyqtSignal() # generic notify signal
+    dataChanged = pyqtSignal()  # generic notify signal
+
+    def __init__(self, parent=None):
+        super().__init__(parent)
+
+        self._wallet = None  # type: Optional[QEWallet]
+        self._connect_str = None
+        self._amount = QEAmount()
+        self._valid = False
+        self._opentx = None
+        self._txdetails = None
+
+        self._finalizer = None
+        self._node_pubkey = None
+        self._connect_str_resolved = None
 
     walletChanged = pyqtSignal()
     @pyqtProperty(QEWallet, notify=walletChanged)
@@ -50,17 +57,17 @@ class QEChannelOpener(QObject, AuthMixin):
             self._wallet = wallet
             self.walletChanged.emit()
 
-    nodeidChanged = pyqtSignal()
-    @pyqtProperty(str, notify=nodeidChanged)
-    def nodeid(self):
-        return self._nodeid
+    connectStrChanged = pyqtSignal()
+    @pyqtProperty(str, notify=connectStrChanged)
+    def connectStr(self):
+        return self._connect_str
 
-    @nodeid.setter
-    def nodeid(self, nodeid: str):
-        if self._nodeid != nodeid:
-            self._logger.debug('nodeid set -> %s' % nodeid)
-            self._nodeid = nodeid
-            self.nodeidChanged.emit()
+    @connectStr.setter
+    def connectStr(self, connect_str: str):
+        if self._connect_str != connect_str:
+            self._logger.debug('connectStr set -> %s' % connect_str)
+            self._connect_str = connect_str
+            self.connectStrChanged.emit()
             self.validate()
 
     amountChanged = pyqtSignal()
@@ -71,7 +78,7 @@ class QEChannelOpener(QObject, AuthMixin):
     @amount.setter
     def amount(self, amount: QEAmount):
         if self._amount != amount:
-            self._amount = amount
+            self._amount.copyFrom(amount)
             self.amountChanged.emit()
             self.validate()
 
@@ -97,25 +104,32 @@ class QEChannelOpener(QObject, AuthMixin):
     # FIXME min channel funding amount
     # FIXME have requested funding amount
     def validate(self):
-        nodeid_valid = False
-        if self._nodeid:
-            self._logger.debug(f'checking if {self._nodeid} is valid')
-            if not self._wallet.wallet.config.get('use_gossip', False):
-                self._peer = hardcoded_trampoline_nodes()[self._nodeid]
-                nodeid_valid = True
+        """side-effects: sets self._valid, self._node_pubkey, self._connect_str_resolved"""
+        connect_str_valid = False
+        if self._connect_str:
+            self._logger.debug(f'checking if {self._connect_str=!r} is valid')
+            if not self._wallet.wallet.config.LIGHTNING_USE_GOSSIP:
+                # using trampoline: connect_str is the name of a trampoline node
+                peer_addr = hardcoded_trampoline_nodes()[self._connect_str]
+                self._node_pubkey = peer_addr.pubkey
+                self._connect_str_resolved = str(peer_addr)
+                connect_str_valid = True
             else:
+                # using gossip: connect_str is anything extract_nodeid() can parse
                 try:
-                    self._peer = self.nodeid_to_lnpeer(self._nodeid)
-                    nodeid_valid = True
-                except:
+                    self._node_pubkey, _rest = extract_nodeid(self._connect_str)
+                except ConnStringFormatError:
                     pass
+                else:
+                    self._connect_str_resolved = self._connect_str
+                    connect_str_valid = True
 
-        if not nodeid_valid:
+        if not connect_str_valid:
             self._valid = False
             self.validChanged.emit()
             return
 
-        self._logger.debug('amount=%s' % str(self._amount))
+        self._logger.debug(f'amount={self._amount}')
         if not self._amount or not (self._amount.satsInt > 0 or self._amount.isMax):
             self._valid = False
             self.validChanged.emit()
@@ -125,30 +139,25 @@ class QEChannelOpener(QObject, AuthMixin):
         self.validChanged.emit()
 
     @pyqtSlot(str, result=bool)
-    def validate_nodeid(self, nodeid):
+    def validateConnectString(self, connect_str):
         try:
-            self.nodeid_to_lnpeer(nodeid)
-        except Exception as e:
-            self._logger.debug(repr(e))
+            extract_nodeid(connect_str)
+        except ConnStringFormatError as e:
+            self._logger.debug(f'invalid connect_str. {e!r}')
             return False
         return True
 
-    def nodeid_to_lnpeer(self, nodeid):
-        node_pubkey, host_port = extract_nodeid(nodeid)
-        host, port = host_port.split(':',1)
-        return LNPeerAddr(host, int(port), node_pubkey)
-
-    # FIXME "max" button in amount_dialog should enforce LN_MAX_FUNDING_SAT
+    # FIXME "max" button in amount_dialog should enforce LIGHTNING_MAX_FUNDING_SAT
     @pyqtSlot()
     @pyqtSlot(bool)
-    def open_channel(self, confirm_backup_conflict=False):
+    def openChannel(self, confirm_backup_conflict=False):
         if not self.valid:
             return
 
-        self._logger.debug('Connect String: %s' % str(self._peer))
+        self._logger.debug(f'Connect String: {self._connect_str!r}')
 
         lnworker = self._wallet.wallet.lnworker
-        if lnworker.has_conflicting_backup_with(self._peer.pubkey) and not confirm_backup_conflict:
+        if lnworker.has_conflicting_backup_with(self._node_pubkey) and not confirm_backup_conflict:
             self.conflictingBackup.emit(messages.MGS_CONFLICTING_BACKUP_INSTANCE)
             return
 
@@ -160,10 +169,10 @@ class QEChannelOpener(QObject, AuthMixin):
         mktx = lambda amt: lnworker.mktx_for_open_channel(
             coins=coins,
             funding_sat=amt,
-            node_id=self._peer.pubkey,
+            node_id=self._node_pubkey,
             fee_est=None)
 
-        acpt = lambda tx: self.do_open_channel(tx, str(self._peer), None)
+        acpt = lambda tx: self.do_open_channel(tx, self._connect_str_resolved, self._wallet.password)
 
         self._finalizer = QETxFinalizer(self, make_tx=mktx, accept=acpt)
         self._finalizer.canRbf = False
@@ -171,11 +180,14 @@ class QEChannelOpener(QObject, AuthMixin):
         self._finalizer.wallet = self._wallet
         self.finalizerChanged.emit()
 
-    @auth_protect
+    @auth_protect(message=_('Open Lightning channel?'))
     def do_open_channel(self, funding_tx, conn_str, password):
+        """
+        conn_str: a connection string that extract_nodeid can parse, i.e. cannot be a trampoline name
+        """
         self._logger.debug('opening channel')
         # read funding_sat from tx; converts '!' to int value
-        funding_sat = funding_tx.output_value_for_address(ln_dummy_address())
+        funding_sat = funding_tx.output_value_for_address(DummyAddress.CHANNEL)
         lnworker = self._wallet.wallet.lnworker
 
         def open_thread():
@@ -192,13 +204,13 @@ class QEChannelOpener(QObject, AuthMixin):
                                              chan.constraints.funding_txn_minimum_depth, funding_tx.is_complete())
 
                 # TODO: handle incomplete TX
-                #if not funding_tx.is_complete():
-                    #self._txdetails = QETxDetails(self)
-                    #self._txdetails.rawTx = funding_tx
-                    #self._txdetails.wallet = self._wallet
-                    #self.txDetailsChanged.emit()
+                # if not funding_tx.is_complete():
+                #     self._txdetails = QETxDetails(self)
+                #     self._txdetails.rawTx = funding_tx
+                #     self._txdetails.wallet = self._wallet
+                #     self.txDetailsChanged.emit()
 
-            except (CancelledError,TimeoutError):
+            except (CancelledError, TimeoutError):
                 error = _('Could not connect to channel peer')
             except Exception as e:
                 error = str(e)
@@ -209,41 +221,9 @@ class QEChannelOpener(QObject, AuthMixin):
                     self._logger.exception("Problem opening channel: %s", error)
                     self.channelOpenError.emit(error)
 
-
         self._logger.debug('starting open thread')
         self.channelOpening.emit(conn_str)
-        threading.Thread(target=open_thread).start()
-
-        # TODO: it would be nice to show this before broadcasting
-        #if chan.has_onchain_backup():
-            #self.maybe_show_funding_tx(chan, funding_tx)
-        #else:
-            #title = _('Save backup')
-            #help_text = _(messages.MSG_CREATED_NON_RECOVERABLE_CHANNEL)
-            #data = lnworker.export_channel_backup(chan.channel_id)
-            #popup = QRDialog(
-                #title, data,
-                #show_text=False,
-                #text_for_clipboard=data,
-                #help_text=help_text,
-                #close_button_text=_('OK'),
-                #on_close=lambda: self.maybe_show_funding_tx(chan, funding_tx))
-            #popup.open()
-
-
-    #def maybe_show_funding_tx(self, chan, funding_tx):
-        #n = chan.constraints.funding_txn_minimum_depth
-        #message = '\n'.join([
-            #_('Channel established.'),
-            #_('Remote peer ID') + ':' + chan.node_id.hex(),
-            #_('This channel will be usable after {} confirmations').format(n)
-        #])
-        #if not funding_tx.is_complete():
-            #message += '\n\n' + _('Please sign and broadcast the funding transaction')
-        #self.app.show_info(message)
-
-        #if not funding_tx.is_complete():
-            #self.app.tx_dialog(funding_tx)
+        threading.Thread(target=open_thread, daemon=True).start()
 
     @pyqtSlot(str, result=str)
     def channelBackup(self, cid):
