@@ -1,39 +1,33 @@
-import threading
-from typing import TYPE_CHECKING, Optional, Dict, Any
-import asyncio
+from enum import IntEnum
+from typing import Optional, Dict, Any
 from urllib.parse import urlparse
 
-from PyQt5.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject, Q_ENUMS, QTimer
+from PyQt6.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject, pyqtEnum, QTimer
 
-from electrum import bitcoin
-from electrum import lnutil
 from electrum.i18n import _
-from electrum.invoices import Invoice
-from electrum.invoices import (PR_UNPAID, PR_EXPIRED, PR_UNKNOWN, PR_PAID, PR_INFLIGHT,
-                               PR_FAILED, PR_ROUTING, PR_UNCONFIRMED, PR_BROADCASTING, PR_BROADCAST, LN_EXPIRY_NEVER)
-from electrum.lnaddr import LnInvoiceException
 from electrum.logging import get_logger
-from electrum.transaction import PartialTxOutput
-from electrum.util import (parse_URI, InvalidBitcoinURI, InvoiceError,
-                           maybe_extract_lightning_payment_identifier, get_asyncio_loop)
+from electrum.invoices import (Invoice, PR_UNPAID, PR_EXPIRED, PR_UNKNOWN, PR_PAID, PR_INFLIGHT,
+                               PR_FAILED, PR_ROUTING, PR_UNCONFIRMED, PR_BROADCASTING, PR_BROADCAST, LN_EXPIRY_NEVER)
+from electrum.transaction import PartialTxOutput, TxOutput
 from electrum.lnutil import format_short_channel_id
-from electrum.lnurl import decode_lnurl, request_lnurl, callback_lnurl
 from electrum.bitcoin import COIN
 from electrum.paymentrequest import PaymentRequest
-
+from electrum.payment_identifier import (PaymentIdentifier, PaymentIdentifierState, PaymentIdentifierType)
 from .qetypes import QEAmount
 from .qewallet import QEWallet
 from .util import status_update_timer_interval, QtEventListener, event_listener
 
 
 class QEInvoice(QObject, QtEventListener):
-    class Type:
+    @pyqtEnum
+    class Type(IntEnum):
         Invalid = -1
         OnchainInvoice = 0
         LightningInvoice = 1
         LNURLPayRequest = 2
 
-    class Status:
+    @pyqtEnum
+    class Status(IntEnum):
         Unpaid = PR_UNPAID
         Expired = PR_EXPIRED
         Unknown = PR_UNKNOWN
@@ -43,9 +37,6 @@ class QEInvoice(QObject, QtEventListener):
         Routing = PR_ROUTING
         Unconfirmed = PR_UNCONFIRMED
 
-    Q_ENUMS(Type)
-    Q_ENUMS(Status)
-
     _logger = get_logger(__name__)
 
     invoiceChanged = pyqtSignal()
@@ -100,6 +91,11 @@ class QEInvoice(QObject, QtEventListener):
             self.determine_can_pay()
             self.statusChanged.emit()
 
+    @event_listener
+    def on_event_channel(self, wallet, channel):
+        if self._wallet and wallet == self._wallet.wallet:
+            self.determine_can_pay()
+
     walletChanged = pyqtSignal()
     @pyqtProperty(QEWallet, notify=walletChanged)
     def wallet(self):
@@ -249,7 +245,8 @@ class QEInvoice(QObject, QtEventListener):
         }
 
     def name_for_node_id(self, node_id):
-        return self._wallet.wallet.lnworker.get_node_alias(node_id) or node_id.hex()
+        lnworker = self._wallet.wallet.lnworker
+        return (lnworker.get_node_alias(node_id) if lnworker else None) or node_id.hex()
 
     def set_effective_invoice(self, invoice: Invoice):
         self._effectiveInvoice = invoice
@@ -283,7 +280,7 @@ class QEInvoice(QObject, QtEventListener):
                     self._timer.start()
         else:
             self.update_userinfo()
-            self.determine_can_pay() # status went to PR_EXPIRED
+            self.determine_can_pay()  # status went to PR_EXPIRED
 
     @pyqtSlot()
     def updateStatusString(self):
@@ -301,7 +298,7 @@ class QEInvoice(QObject, QtEventListener):
         if self.amount.isEmpty:
             self.userinfo = _('Enter the amount you want to send')
 
-        if amount.isEmpty and self.status == PR_UNPAID: # unspecified amount
+        if amount.isEmpty and self.status == PR_UNPAID:  # unspecified amount
             return
 
         if self.invoiceType == QEInvoice.Type.LightningInvoice:
@@ -330,9 +327,9 @@ class QEInvoice(QObject, QtEventListener):
                 self.userinfo = {
                         PR_EXPIRED: _('This invoice has expired'),
                         PR_PAID: _('This invoice was already paid'),
-                        PR_BROADCASTING: _('Payment in progress...') + ' (' +  _('broadcasting') + ')',
-                        PR_BROADCAST:  _('Payment in progress...') + ' (' +  _('broadcast successfully') + ')',
-                        PR_UNCONFIRMED: _('Payment in progress...') + ' (' +  _('waiting for confirmation') + ')',
+                        PR_BROADCASTING: _('Payment in progress...') + ' (' + _('broadcasting') + ')',
+                        PR_BROADCAST:  _('Payment in progress...') + ' (' + _('broadcast successfully') + ')',
+                        PR_UNCONFIRMED: _('Payment in progress...') + ' (' + _('waiting for confirmation') + ')',
                         PR_UNKNOWN: _('Invoice has unknown status'),
                     }[self.status]
 
@@ -340,14 +337,17 @@ class QEInvoice(QObject, QtEventListener):
         self.canPay = False
         self.canSave = False
 
+        if self.invoiceType not in [QEInvoice.Type.LightningInvoice, QEInvoice.Type.OnchainInvoice]:
+            return
+
         if not self.amountOverride.isEmpty:
             amount = self.amountOverride
         else:
             amount = self.amount
 
-        self.canSave = True
+        self.canSave = not bool(self._wallet.wallet.get_invoice(self._effectiveInvoice.get_id()))
 
-        if amount.isEmpty and self.status == PR_UNPAID: # unspecified amount
+        if amount.isEmpty and self.status == PR_UNPAID:  # unspecified amount
             return
 
         if self.invoiceType == QEInvoice.Type.LightningInvoice:
@@ -380,39 +380,41 @@ class QEInvoice(QObject, QtEventListener):
         if self.amount.isEmpty:
             if self.amountOverride.isEmpty:
                 raise Exception('can not pay 0 amount')
-            # TODO: is update amount_msat for overrideAmount sufficient?
-            self._effectiveInvoice.amount_msat = self.amountOverride.satsInt * 1000
+            self._effectiveInvoice.set_amount_msat(self.amountOverride.satsInt * 1000)
 
         self._wallet.pay_lightning_invoice(self._effectiveInvoice)
 
     def get_max_spendable_onchain(self):
         spendable = self._wallet.confirmedBalance.satsInt
-        if not self._wallet.wallet.config.get('confirmed_only', False):
+        if not self._wallet.wallet.config.WALLET_SPEND_CONFIRMED_ONLY:
             spendable += self._wallet.unconfirmedBalance.satsInt
         return spendable
 
     def get_max_spendable_lightning(self):
         return self._wallet.wallet.lnworker.num_sats_can_send() if self._wallet.wallet.lnworker else 0
 
+
 class QEInvoiceParser(QEInvoice):
     _logger = get_logger(__name__)
 
     validationSuccess = pyqtSignal()
-    validationWarning = pyqtSignal([str,str], arguments=['code', 'message'])
-    validationError = pyqtSignal([str,str], arguments=['code', 'message'])
+    validationWarning = pyqtSignal([str, str], arguments=['code', 'message'])
+    validationError = pyqtSignal([str, str], arguments=['code', 'message'])
 
-    invoiceCreateError = pyqtSignal([str,str], arguments=['code', 'message'])
+    invoiceCreateError = pyqtSignal([str, str], arguments=['code', 'message'])
 
     lnurlRetrieved = pyqtSignal()
-    lnurlError = pyqtSignal([str,str], arguments=['code', 'message'])
+    lnurlError = pyqtSignal([str, str], arguments=['code', 'message'])
 
-    _bip70PrResolvedSignal = pyqtSignal([PaymentRequest], arguments=['pr'])
+    busyChanged = pyqtSignal()
 
     def __init__(self, parent=None):
         super().__init__(parent)
 
         self._recipient = ''
-        self._bip70PrResolvedSignal.connect(self._bip70_payment_request_resolved)
+        self._pi = None
+        self._lnurlData = None
+        self._busy = False
 
         self.clear()
 
@@ -438,6 +440,10 @@ class QEInvoiceParser(QEInvoice):
     def isLnurlPay(self):
         return self._lnurlData is not None
 
+    @pyqtProperty(bool, notify=busyChanged)
+    def busy(self):
+        return self._busy
+
     @pyqtSlot()
     def clear(self):
         self.recipient = ''
@@ -476,7 +482,7 @@ class QEInvoiceParser(QEInvoice):
 
     def _bip70_payment_request_resolved(self, pr: 'PaymentRequest'):
         self._logger.debug('resolved payment request')
-        if pr.verify(self._wallet.wallet.contacts):
+        if pr.verify():
             invoice = Invoice.from_bip70_payreq(pr, height=0)
             if self._wallet.wallet.get_invoice_status(invoice) == PR_PAID:
                 self.validationError.emit('unknown', _('Invoice already paid'))
@@ -486,114 +492,109 @@ class QEInvoiceParser(QEInvoice):
                 self.setValidOnchainInvoice(invoice)
                 self.validationSuccess.emit()
         else:
-            self.validationError.emit('unknown', f"invoice error:\n{pr.error}")
+            self.validationError.emit('unknown', f'invoice error:\n{pr.error}')
 
     def validateRecipient(self, recipient):
         if not recipient:
             self.setInvoiceType(QEInvoice.Type.Invalid)
             return
 
-        maybe_lightning_invoice = recipient
-
-        try:
-            bip21 = parse_URI(recipient, lambda pr: self._bip70PrResolvedSignal.emit(pr))
-            if bip21:
-                if 'r' in bip21 or ('name' in bip21 and 'sig' in bip21): # TODO set flag in util?
-                    # let callback handle state
-                    return
-                if ':' not in recipient:
-                    # address only
-                    # create bare invoice
-                    outputs = [PartialTxOutput.from_address_and_value(bip21['address'], 0)]
-                    invoice = self.create_onchain_invoice(outputs, None, None, None)
-                    self._logger.debug(repr(invoice))
-                    self.setValidOnchainInvoice(invoice)
-                    self.validationSuccess.emit()
-                    return
-                else:
-                    # fallback lightning invoice?
-                    if 'lightning' in bip21:
-                        maybe_lightning_invoice = bip21['lightning']
-        except InvalidBitcoinURI as e:
-            bip21 = None
-
-        lninvoice = None
-        maybe_lightning_invoice = maybe_extract_lightning_payment_identifier(maybe_lightning_invoice)
-        if maybe_lightning_invoice is not None:
-            if maybe_lightning_invoice.startswith('lnurl'):
-                self.resolve_lnurl(maybe_lightning_invoice)
-                return
-            try:
-                lninvoice = Invoice.from_bech32(maybe_lightning_invoice)
-            except InvoiceError as e:
-                e2 = e.__cause__
-                if isinstance(e2, LnInvoiceException):
-                    self.validationError.emit('unknown', _("Error parsing Lightning invoice") + f":\n{e2}")
-                    self.clear()
-                    return
-                if isinstance(e2, lnutil.IncompatibleOrInsaneFeatures):
-                    self.validationError.emit('unknown', _("Invoice requires unknown or incompatible Lightning feature") + f":\n{e2!r}")
-                    self.clear()
-                    return
-                self._logger.exception(repr(e))
-
-        if not lninvoice and not bip21:
-            self.validationError.emit('unknown',_('Unknown invoice'))
-            self.clear()
+        self._pi = PaymentIdentifier(self._wallet.wallet, recipient)
+        if not self._pi.is_valid() or self._pi.type not in [PaymentIdentifierType.SPK, PaymentIdentifierType.BIP21,
+                                                            PaymentIdentifierType.BIP70, PaymentIdentifierType.BOLT11,
+                                                            PaymentIdentifierType.LNURLP,
+                                                            PaymentIdentifierType.EMAILLIKE,
+                                                            PaymentIdentifierType.DOMAINLIKE]:
+            self.validationError.emit('unknown', _('Unknown invoice'))
             return
 
-        if lninvoice:
-            if not self._wallet.wallet.has_lightning():
-                if not bip21:
-                    if lninvoice.get_address():
-                        self.setValidLightningInvoice(lninvoice)
-                        self.validationSuccess.emit()
-                    else:
-                        self.validationError.emit('no_lightning',_('Detected valid Lightning invoice, but Lightning not enabled for wallet and no fallback address found.'))
-                else:
-                    self._logger.debug('flow with LN but not LN enabled AND having bip21 uri')
-                    self._validateRecipient_bip21_onchain(bip21)
-            else:
-                if not self._wallet.wallet.lnworker.channels:
-                    if bip21 and 'address' in bip21:
-                        self._logger.debug('flow where invoice has both LN and onchain, we have LN enabled but no channels')
-                        self._validateRecipient_bip21_onchain(bip21)
-                    else:
-                        self.validationWarning.emit('no_channels',_('Detected valid Lightning invoice, but there are no open channels'))
-                else:
+        if self._pi.type == PaymentIdentifierType.SPK:
+            txo = TxOutput(scriptpubkey=self._pi.spk, value=0)
+            if not txo.address:
+                self.validationError.emit('unknown', _('Unknown invoice'))
+                return
+
+        self._update_from_payment_identifier()
+
+    def _update_from_payment_identifier(self):
+        if self._pi.need_resolve():
+            self.resolve_pi()
+            return
+
+        if self._pi.type in [PaymentIdentifierType.LNURLP, PaymentIdentifierType.LNADDR]:
+            self.on_lnurl(self._pi.lnurl_data)
+            return
+
+        if self._pi.type == PaymentIdentifierType.BIP70:
+            self._bip70_payment_request_resolved(self._pi.bip70_data)
+            return
+
+        if self._pi.is_available():
+            if self._pi.type in [PaymentIdentifierType.SPK, PaymentIdentifierType.OPENALIAS]:
+                outputs = [PartialTxOutput(scriptpubkey=self._pi.spk, value=0)]
+                invoice = self.create_onchain_invoice(outputs, None, None, None)
+                self._logger.debug(repr(invoice))
+                self.setValidOnchainInvoice(invoice)
+                self.validationSuccess.emit()
+                return
+            elif self._pi.type == PaymentIdentifierType.BOLT11:
+                lninvoice = self._pi.bolt11
+                if not self._wallet.wallet.has_lightning() and not lninvoice.get_address():
+                    self.validationError.emit('no_lightning',
+                        _('Detected valid Lightning invoice, but Lightning not enabled for wallet and no fallback address found.'))
+                    return
+                if self._wallet.wallet.lnworker and not self._wallet.wallet.lnworker.channels and not lninvoice.get_address():
+                    self.validationWarning.emit('no_channels',
+                        _('Detected valid Lightning invoice, but there are no open channels'))
+                self.setValidLightningInvoice(lninvoice)
+                self.validationSuccess.emit()
+            elif self._pi.type == PaymentIdentifierType.BIP21:
+                if self._wallet.wallet.has_lightning() and self._wallet.wallet.lnworker.channels and self._pi.bolt11:
+                    lninvoice = self._pi.bolt11
                     self.setValidLightningInvoice(lninvoice)
                     self.validationSuccess.emit()
-        else:
-            self._logger.debug('flow without LN but having bip21 uri')
-            self._validateRecipient_bip21_onchain(bip21)
+                else:
+                    self._validateRecipient_bip21_onchain(self._pi.bip21)
 
     def _validateRecipient_bip21_onchain(self, bip21: Dict[str, Any]) -> None:
-        if 'amount' not in bip21:
-            amount = 0
-        else:
-            amount = bip21['amount']
+        if 'address' not in bip21:
+            self._logger.debug('Neither LN invoice nor address in bip21 uri')
+            self.validationError.emit('unknown', _('Unknown invoice'))
+            return
+
+        amount = bip21.get('amount', 0)
         outputs = [PartialTxOutput.from_address_and_value(bip21['address'], amount)]
         self._logger.debug(outputs)
-        message = bip21['message'] if 'message' in bip21 else ''
+        message = bip21.get('message', '')
         invoice = self.create_onchain_invoice(outputs, message, None, bip21)
         self._logger.debug(repr(invoice))
         self.setValidOnchainInvoice(invoice)
         self.validationSuccess.emit()
 
-    def resolve_lnurl(self, lnurl):
-        self._logger.debug('resolve_lnurl')
-        url = decode_lnurl(lnurl)
-        self._logger.debug(f'{repr(url)}')
+    def resolve_pi(self):
+        assert self._pi.need_resolve()
 
-        def resolve_task():
-            try:
-                coro = request_lnurl(url)
-                fut = asyncio.run_coroutine_threadsafe(coro, get_asyncio_loop())
-                self.on_lnurl(fut.result())
-            except Exception as e:
-                self.validationError.emit('lnurl', repr(e))
+        def on_finished(pi):
+            self._busy = False
+            self.busyChanged.emit()
 
-        threading.Thread(target=resolve_task, daemon=True).start()
+            if pi.is_error():
+                if pi.type in [PaymentIdentifierType.EMAILLIKE, PaymentIdentifierType.DOMAINLIKE]:
+                    msg = _('Could not resolve address')
+                elif pi.type == PaymentIdentifierType.LNURLP:
+                    msg = _('Could not resolve LNURL')
+                elif pi.type == PaymentIdentifierType.BIP70:
+                    msg = _('Could not resolve BIP70 payment request: {}').format(pi.error)
+                else:
+                    msg = _('Could not resolve')
+                self.validationError.emit('resolve', msg)
+            else:
+                self._update_from_payment_identifier()
+
+        self._busy = True
+        self.busyChanged.emit()
+
+        self._pi.resolve(on_finished=on_finished)
 
     def on_lnurl(self, lnurldata):
         self._logger.debug('on_lnurl')
@@ -601,7 +602,7 @@ class QEInvoiceParser(QEInvoice):
 
         self._lnurlData = {
             'domain': urlparse(lnurldata.callback_url).netloc,
-            'callback_url' : lnurldata.callback_url,
+            'callback_url': lnurldata.callback_url,
             'min_sendable_sat': lnurldata.min_sendable_sat,
             'max_sendable_sat': lnurldata.max_sendable_sat,
             'metadata_plaintext': lnurldata.metadata_plaintext,
@@ -610,49 +611,44 @@ class QEInvoiceParser(QEInvoice):
         self.setValidLNURLPayRequest()
         self.lnurlRetrieved.emit()
 
-    @pyqtSlot('quint64')
-    @pyqtSlot('quint64', str)
-    def lnurlGetInvoice(self, amount, comment=None):
+    @pyqtSlot()
+    @pyqtSlot(str)
+    def lnurlGetInvoice(self, comment=None):
         assert self._lnurlData
+        assert self._pi.need_finalize()
         self._logger.debug(f'{repr(self._lnurlData)}')
 
         amount = self.amountOverride.satsInt
-        if self.lnurlData['min_sendable_sat'] != 0:
-            try:
-                assert amount >= self.lnurlData['min_sendable_sat']
-                assert amount <= self.lnurlData['max_sendable_sat']
-            except Exception:
-                self.lnurlError.emit('amount', _('Amount out of bounds'))
-                return
 
         if self._lnurlData['comment_allowed'] == 0:
             comment = None
 
-        self._logger.debug(f'fetching callback url {self._lnurlData["callback_url"]}')
-        def fetch_invoice_task():
-            try:
-                params = { 'amount': amount * 1000 }
-                if comment:
-                    params['comment'] = comment
-                coro = callback_lnurl(self._lnurlData['callback_url'], params)
-                fut = asyncio.run_coroutine_threadsafe(coro, get_asyncio_loop())
-                self.on_lnurl_invoice(amount, fut.result())
-            except Exception as e:
-                self._logger.error(repr(e))
-                self.lnurlError.emit('lnurl', str(e))
+        def on_finished(pi):
+            self._busy = False
+            self.busyChanged.emit()
 
-        threading.Thread(target=fetch_invoice_task, daemon=True).start()
+            if pi.is_error():
+                if pi.state == PaymentIdentifierState.INVALID_AMOUNT:
+                    self.lnurlError.emit('amount', pi.get_error())
+                else:
+                    self.lnurlError.emit('lnurl', pi.get_error())
+            else:
+                self.on_lnurl_invoice(self.amountOverride.satsInt, pi.bolt11)
+
+        self._busy = True
+        self.busyChanged.emit()
+
+        self._pi.finalize(amount_sat=amount, comment=comment, on_finished=on_finished)
 
     def on_lnurl_invoice(self, orig_amount, invoice):
         self._logger.debug('on_lnurl_invoice')
         self._logger.debug(f'{repr(invoice)}')
 
         # assure no shenanigans with the bolt11 invoice we get back
-        lninvoice = Invoice.from_bech32(invoice['pr'])
-        if orig_amount * 1000 != lninvoice.amount_msat:
+        if orig_amount * 1000 != invoice.amount_msat:  # TODO msat precision can cause trouble here
             raise Exception('Unexpected amount in invoice, differs from lnurl-pay specified amount')
 
-        self.recipient = invoice['pr']
+        self.recipient = invoice.lightning_invoice
 
     @pyqtSlot()
     def saveInvoice(self):
@@ -663,9 +659,9 @@ class QEInvoiceParser(QEInvoice):
 
         if not self._effectiveInvoice.amount_msat and not self.amountOverride.isEmpty:
             if self.invoiceType == QEInvoice.Type.OnchainInvoice and self.amountOverride.isMax:
-                self._effectiveInvoice.amount_msat = '!'
+                self._effectiveInvoice.set_amount_msat('!')
             else:
-                self._effectiveInvoice.amount_msat = self.amountOverride.satsInt * 1000
+                self._effectiveInvoice.set_amount_msat(self.amountOverride.satsInt * 1000)
 
         self.canSave = False
 
