@@ -5,7 +5,7 @@ import PyQt5.QtWidgets as QtWidgets
 import PyQt5.QtCore as QtCore
 from PyQt5.QtWidgets import QLabel, QLineEdit, QHBoxLayout, QGridLayout
 
-from electrum.util import EventListener
+from electrum.util import EventListener, ShortID
 from electrum.i18n import _
 from electrum.util import format_time
 from electrum.lnutil import format_short_channel_id, LOCAL, REMOTE, UpdateAddHtlc, Direction
@@ -15,7 +15,7 @@ from electrum.bitcoin import COIN
 from electrum.wallet import Abstract_Wallet
 
 from .util import Buttons, CloseButton, ShowQRLineEdit, MessageBoxMixin, WWLabel
-from .util import QtEventListener, qt_event_listener
+from .util import QtEventListener, qt_event_listener, VLine
 
 if TYPE_CHECKING:
     from .main_window import ElectrumWindow
@@ -85,7 +85,7 @@ class ChannelDetailsDialog(QtWidgets.QDialog, MessageBoxMixin, QtEventListener):
     def make_htlc_item(self, i: UpdateAddHtlc, direction: Direction) -> HTLCItem:
         it = HTLCItem(_('Sent HTLC with ID {}' if Direction.SENT == direction else 'Received HTLC with ID {}').format(i.htlc_id))
         it.appendRow([HTLCItem(_('Amount')),HTLCItem(self.format_msat(i.amount_msat))])
-        it.appendRow([HTLCItem(_('CLTV expiry')),HTLCItem(str(i.cltv_expiry))])
+        it.appendRow([HTLCItem(_('CLTV expiry')), HTLCItem(str(i.cltv_abs))])
         it.appendRow([HTLCItem(_('Payment hash')),HTLCItem(i.payment_hash.hex())])
         return it
 
@@ -159,12 +159,14 @@ class ChannelDetailsDialog(QtWidgets.QDialog, MessageBoxMixin, QtEventListener):
     def update(self):
         if self.chan.is_closed() or self.chan.is_backup():
             return
+        assert isinstance(self.chan, Channel), type(self.chan)
         self.can_send_label.setText(self.format_msat(self.chan.available_to_spend(LOCAL)))
         self.can_receive_label.setText(self.format_msat(self.chan.available_to_spend(REMOTE)))
         self.sent_label.setText(self.format_msat(self.chan.total_msat(Direction.SENT)))
         self.received_label.setText(self.format_msat(self.chan.total_msat(Direction.RECEIVED)))
         self.local_balance_label.setText(self.format_msat(self.chan.balance(LOCAL)))
         self.remote_balance_label.setText(self.format_msat(self.chan.balance(REMOTE)))
+        self.current_feerate.setText(self.window.format_fee_rate(4 * self.chan.get_latest_feerate(LOCAL)))
 
     @QtCore.pyqtSlot(str)
     def show_tx(self, link_text: str):
@@ -180,10 +182,11 @@ class ChannelDetailsDialog(QtWidgets.QDialog, MessageBoxMixin, QtEventListener):
         form.addRow(QLabel(_('Remote Node') + ':'), remote_id_e)
         channel_id_e = ShowQRLineEdit(chan.channel_id.hex(), self.window.config, title=_("Channel ID"))
         form.addRow(QLabel(_('Channel ID') + ':'), channel_id_e)
-        form.addRow(QLabel(_('Short Channel ID') + ':'), QLabel(str(chan.short_channel_id)))
-        alias = chan.get_remote_alias()
-        if alias:
-            form.addRow(QLabel(_('Alias') + ':'), QLabel('0x'+alias.hex()))
+        form.addRow(QLabel(_('Short Channel ID') + ':'), SelectableLabel(str(chan.short_channel_id)))
+        if local_scid_alias := chan.get_local_scid_alias():
+            form.addRow(QLabel('Local SCID Alias:'), SelectableLabel(str(ShortID(local_scid_alias))))
+        if remote_scid_alias := chan.get_remote_scid_alias():
+            form.addRow(QLabel('Remote SCID Alias:'), SelectableLabel(str(ShortID(remote_scid_alias))))
         form.addRow(QLabel(_('State') + ':'), SelectableLabel(chan.get_state_for_GUI()))
         self.capacity = self.format_sat(chan.get_capacity())
         form.addRow(QLabel(_('Capacity') + ':'), SelectableLabel(self.capacity))
@@ -204,7 +207,7 @@ class ChannelDetailsDialog(QtWidgets.QDialog, MessageBoxMixin, QtEventListener):
                 form.addRow(QLabel(_('Closing Transaction') + ':'), LinkedLabel(closing_label_text, self.show_tx))
         return form
 
-    def get_hbox_stats(self, chan):
+    def get_hbox_stats(self, chan: Channel):
         hbox_stats = QHBoxLayout()
         form_layout_left = QtWidgets.QFormLayout(None)
         form_layout_right = QtWidgets.QFormLayout(None)
@@ -238,14 +241,22 @@ class ChannelDetailsDialog(QtWidgets.QDialog, MessageBoxMixin, QtEventListener):
         self.sent_label = SelectableLabel()
         form_layout_left.addRow(_('Total sent') + ':', self.sent_label)
         form_layout_right.addRow(_('Total received') + ':', self.received_label)
+        # to-self-delay
+        csv_local_header = SelectableLabel(_("Remote force-close CSV delay") + ":")
+        csv_local_header.setToolTip(_("Force-close CSV delay imposed on them"))
+        csv_remote_header = SelectableLabel(_("Local force-close CSV delay") + ":")
+        csv_remote_header.setToolTip(_("Force-close CSV delay imposed on us"))
+        csv_local_label  = SelectableLabel(_("{} blocks").format(chan.config[LOCAL].to_self_delay))
+        csv_remote_label = SelectableLabel(_("{} blocks").format(chan.config[REMOTE].to_self_delay))
+        form_layout_left.addRow(csv_local_header, csv_local_label)
+        form_layout_right.addRow(csv_remote_header, csv_remote_label)
+        # onchain feerate
+        self.current_feerate = SelectableLabel()
+        form_layout_left.addRow(_('Current feerate') + ':', self.current_feerate)
         # channel stats left column
         hbox_stats.addLayout(form_layout_left, 50)
         # vertical line separator
-        line_separator = QtWidgets.QFrame()
-        line_separator.setFrameShape(QtWidgets.QFrame.VLine)
-        line_separator.setFrameShadow(QtWidgets.QFrame.Sunken)
-        line_separator.setLineWidth(1)
-        hbox_stats.addWidget(line_separator)
+        hbox_stats.addWidget(VLine())
         # channel stats right column
         hbox_stats.addLayout(form_layout_right, 50)
         return hbox_stats
