@@ -27,6 +27,7 @@
 from unicodedata import normalize
 import hashlib
 import re
+import copy
 from typing import Tuple, TYPE_CHECKING, Union, Sequence, Optional, Dict, List, NamedTuple
 from functools import lru_cache, wraps
 from abc import ABC, abstractmethod
@@ -58,6 +59,8 @@ if TYPE_CHECKING:
 
 class CannotDerivePubkey(Exception): pass
 
+class ScriptTypeNotSupported(Exception): pass
+
 
 def also_test_none_password(check_password_fn):
     """Decorator for check_password, simply to give a friendlier exception if
@@ -1067,7 +1070,9 @@ def hardware_keystore(d) -> Hardware_KeyStore:
                               f'hw_keystores: {list(hw_keystores)}')
 
 def load_keystore(db: 'WalletDB', name: str) -> KeyStore:
-    d = db.get(name, {})
+    # deepcopy object to avoid keeping a pointer to db.data
+    # note: this is needed as type(wallet.db.get("keystore")) != StoredDict
+    d = copy.deepcopy(db.get(name, {}))
     t = d.get('type')
     if not t:
         raise WalletFileException(
