@@ -7,7 +7,9 @@ from collections import OrderedDict
 from .lnutil import OnionFailureCodeMetaFlag
 
 
-class FailedToParseMsg(Exception): pass
+class FailedToParseMsg(Exception):
+    msg_type_int: Optional[int] = None
+    msg_type_name: Optional[str] = None
 
 class UnknownMsgType(FailedToParseMsg): pass
 class UnknownOptionalMsgType(UnknownMsgType): pass
@@ -127,7 +129,7 @@ def _read_field(*, fd: io.BytesIO, field_type: str, count: Union[int, str]) -> U
         if len(raw) > 0 and raw[0] == 0x00:
             raise FieldEncodingNotMinimal()
         return int.from_bytes(raw, byteorder="big", signed=False)
-    elif field_type == 'varint':
+    elif field_type == 'bigsize':
         assert count == 1, count
         val = read_bigsize_int(fd)
         if val is None:
@@ -201,7 +203,7 @@ def _write_field(*, fd: io.BytesIO, field_type: str, count: Union[int, str],
         if nbytes_written != len(value):
             raise Exception(f"tried to write {len(value)} bytes, but only wrote {nbytes_written}!?")
         return
-    elif field_type == 'varint':
+    elif field_type == 'bigsize':
         assert count == 1, count
         if isinstance(value, int):
             value = write_bigsize_int(value)
@@ -241,8 +243,8 @@ def _write_field(*, fd: io.BytesIO, field_type: str, count: Union[int, str],
 
 def _read_tlv_record(*, fd: io.BytesIO) -> Tuple[int, bytes]:
     if not fd: raise Exception()
-    tlv_type = _read_field(fd=fd, field_type="varint", count=1)
-    tlv_len = _read_field(fd=fd, field_type="varint", count=1)
+    tlv_type = _read_field(fd=fd, field_type="bigsize", count=1)
+    tlv_len = _read_field(fd=fd, field_type="bigsize", count=1)
     tlv_val = _read_field(fd=fd, field_type="byte", count=tlv_len)
     return tlv_type, tlv_val
 
@@ -250,8 +252,8 @@ def _read_tlv_record(*, fd: io.BytesIO) -> Tuple[int, bytes]:
 def _write_tlv_record(*, fd: io.BytesIO, tlv_type: int, tlv_val: bytes) -> None:
     if not fd: raise Exception()
     tlv_len = len(tlv_val)
-    _write_field(fd=fd, field_type="varint", count=1, value=tlv_type)
-    _write_field(fd=fd, field_type="varint", count=1, value=tlv_len)
+    _write_field(fd=fd, field_type="bigsize", count=1, value=tlv_type)
+    _write_field(fd=fd, field_type="bigsize", count=1, value=tlv_len)
     _write_field(fd=fd, field_type="byte", count=tlv_len, value=tlv_val)
 
 
@@ -452,10 +454,7 @@ class LNSerializer:
                     try:
                         field_value = kwargs[field_name]
                     except KeyError:
-                        if len(row) > 5:
-                            break  # optional feature field not present
-                        else:
-                            field_value = 0  # default mandatory fields to zero
+                        field_value = 0  # default mandatory fields to zero
                     #print(f">>> encode_msg. writing field: {field_name}. value={field_value!r}. field_type={field_type!r}. count={field_count!r}")
                     _write_field(fd=fd,
                                  field_type=field_type,
@@ -488,33 +487,33 @@ class LNSerializer:
         assert scheme[0][2] == msg_type_int
         msg_type_name = scheme[0][1]
         parsed = {}
-        with io.BytesIO(data[2:]) as fd:
-            for row in scheme:
-                #print(f"row: {row!r}")
-                if row[0] == "msgtype":
-                    pass
-                elif row[0] == "msgdata":
-                    field_name = row[2]
-                    field_type = row[3]
-                    field_count_str = row[4]
-                    field_count = _resolve_field_count(field_count_str, vars_dict=parsed)
-                    if field_name == "tlvs":
-                        tlv_stream_name = field_type
-                        d = self.read_tlv_stream(fd=fd, tlv_stream_name=tlv_stream_name)
-                        parsed[tlv_stream_name] = d
-                        continue
-                    #print(f">> count={field_count}. parsed={parsed}")
-                    try:
-                        parsed[field_name] = _read_field(fd=fd,
-                                                         field_type=field_type,
-                                                         count=field_count)
-                    except UnexpectedEndOfStream as e:
-                        if len(row) > 5:
-                            break  # optional feature field not present
-                        else:
-                            raise
-                else:
-                    raise Exception(f"unexpected row in scheme: {row!r}")
+        try:
+            with io.BytesIO(data[2:]) as fd:
+                for row in scheme:
+                    #print(f"row: {row!r}")
+                    if row[0] == "msgtype":
+                        pass
+                    elif row[0] == "msgdata":
+                        field_name = row[2]
+                        field_type = row[3]
+                        field_count_str = row[4]
+                        field_count = _resolve_field_count(field_count_str, vars_dict=parsed)
+                        if field_name == "tlvs":
+                            tlv_stream_name = field_type
+                            d = self.read_tlv_stream(fd=fd, tlv_stream_name=tlv_stream_name)
+                            parsed[tlv_stream_name] = d
+                            continue
+                        #print(f">> count={field_count}. parsed={parsed}")
+                        parsed[field_name] = _read_field(
+                            fd=fd,
+                            field_type=field_type,
+                            count=field_count)
+                    else:
+                        raise Exception(f"unexpected row in scheme: {row!r}")
+        except FailedToParseMsg as e:
+            e.msg_type_int = msg_type_int
+            e.msg_type_name = msg_type_name
+            raise
         return msg_type_name, parsed
 
 
