@@ -27,6 +27,7 @@ import queue
 from collections import defaultdict
 from typing import Sequence, Tuple, Optional, Dict, TYPE_CHECKING, Set
 import time
+import threading
 from threading import RLock
 import attr
 from math import inf
@@ -34,7 +35,7 @@ from math import inf
 from .util import profiler, with_lock
 from .logging import Logger
 from .lnutil import (NUM_MAX_EDGES_IN_PAYMENT_PATH, ShortChannelID, LnFeatures,
-                     NBLOCK_CLTV_EXPIRY_TOO_FAR_INTO_FUTURE)
+                     NBLOCK_CLTV_DELTA_TOO_FAR_INTO_FUTURE, PaymentFeeBudget)
 from .channel_db import ChannelDB, Policy, NodeInfo
 
 if TYPE_CHECKING:
@@ -42,7 +43,6 @@ if TYPE_CHECKING:
 
 DEFAULT_PENALTY_BASE_MSAT = 500  # how much base fee we apply for unknown sending capability of a channel
 DEFAULT_PENALTY_PROPORTIONAL_MILLIONTH = 100  # how much relative fee we apply for unknown sending capability of a channel
-BLACKLIST_DURATION = 3600  # how long (in seconds) a channel remains blacklisted
 HINT_DURATION = 3600  # how long (in seconds) a liquidity hint remains valid
 
 
@@ -73,10 +73,10 @@ class PathEdge:
 
 @attr.s
 class RouteEdge(PathEdge):
-    fee_base_msat = attr.ib(type=int, kw_only=True)
-    fee_proportional_millionths = attr.ib(type=int, kw_only=True)
-    cltv_expiry_delta = attr.ib(type=int, kw_only=True)
-    node_features = attr.ib(type=int, kw_only=True, repr=lambda val: str(int(val)))  # note: for end node!
+    fee_base_msat = attr.ib(type=int, kw_only=True)                # for start_node
+    fee_proportional_millionths = attr.ib(type=int, kw_only=True)  # for start_node
+    cltv_delta = attr.ib(type=int, kw_only=True)                   # for start_node
+    node_features = attr.ib(type=int, kw_only=True, repr=lambda val: str(int(val)))  # note: for end_node!
 
     def fee_for_edge(self, amount_msat: int) -> int:
         return fee_for_edge_msat(forwarded_amount_msat=amount_msat,
@@ -87,7 +87,7 @@ class RouteEdge(PathEdge):
     def from_channel_policy(
             cls,
             *,
-            channel_policy: 'Policy',
+            channel_policy: 'Policy',  # for start_node
             short_channel_id: bytes,
             start_node: bytes,
             end_node: bytes,
@@ -102,16 +102,16 @@ class RouteEdge(PathEdge):
             short_channel_id=ShortChannelID.normalize(short_channel_id),
             fee_base_msat=channel_policy.fee_base_msat,
             fee_proportional_millionths=channel_policy.fee_proportional_millionths,
-            cltv_expiry_delta=channel_policy.cltv_expiry_delta,
+            cltv_delta=channel_policy.cltv_delta,
             node_features=node_info.features if node_info else 0)
 
     def is_sane_to_use(self, amount_msat: int) -> bool:
         # TODO revise ad-hoc heuristics
         # cltv cannot be more than 2 weeks
-        if self.cltv_expiry_delta > 14 * 144:
+        if self.cltv_delta > 14 * 144:
             return False
         total_fee = self.fee_for_edge(amount_msat)
-        if not is_fee_sane(total_fee, payment_amount_msat=amount_msat):
+        if total_fee > get_default_fee_budget_msat(invoice_amount_msat=amount_msat):
             return False
         return True
 
@@ -135,42 +135,49 @@ class TrampolineEdge(RouteEdge):
 
 LNPaymentPath = Sequence[PathEdge]
 LNPaymentRoute = Sequence[RouteEdge]
+LNPaymentTRoute = Sequence[TrampolineEdge]
 
 
-def is_route_sane_to_use(route: LNPaymentRoute, invoice_amount_msat: int, min_final_cltv_expiry: int) -> bool:
+def is_route_within_budget(
+    route: LNPaymentRoute,
+    *,
+    budget: PaymentFeeBudget,
+    amount_msat_for_dest: int,  # that final receiver gets
+    cltv_delta_for_dest: int,   # that final receiver gets
+) -> bool:
     """Run some sanity checks on the whole route, before attempting to use it.
     called when we are paying; so e.g. lower cltv is better
     """
     if len(route) > NUM_MAX_EDGES_IN_PAYMENT_PATH:
         return False
-    amt = invoice_amount_msat
-    cltv = min_final_cltv_expiry
+    amt = amount_msat_for_dest
+    cltv_cost_of_route = 0  # excluding cltv_delta_for_dest
     for route_edge in reversed(route[1:]):
         if not route_edge.is_sane_to_use(amt): return False
         amt += route_edge.fee_for_edge(amt)
-        cltv += route_edge.cltv_expiry_delta
-    total_fee = amt - invoice_amount_msat
-    # TODO revise ad-hoc heuristics
-    if cltv > NBLOCK_CLTV_EXPIRY_TOO_FAR_INTO_FUTURE:
+        cltv_cost_of_route += route_edge.cltv_delta
+    fee_cost = amt - amount_msat_for_dest
+    # check against budget
+    if cltv_cost_of_route > budget.cltv:
         return False
-    if not is_fee_sane(total_fee, payment_amount_msat=invoice_amount_msat):
+    if fee_cost > budget.fee_msat:
+        return False
+    # sanity check
+    total_cltv_delta = cltv_cost_of_route + cltv_delta_for_dest
+    if total_cltv_delta > NBLOCK_CLTV_DELTA_TOO_FAR_INTO_FUTURE:
         return False
     return True
 
 
-def is_fee_sane(fee_msat: int, *, payment_amount_msat: int) -> bool:
-    # fees <= 5 sat are fine
-    if fee_msat <= 5_000:
-        return True
+def get_default_fee_budget_msat(*, invoice_amount_msat: int) -> int:
     # fees <= 1 % of payment are fine
-    if 100 * fee_msat <= payment_amount_msat:
-        return True
-    return False
+    # fees <= 5 sat are fine
+    return max(5_000, invoice_amount_msat // 100)
 
 
 class LiquidityHint:
     """Encodes the amounts that can and cannot be sent over the direction of a
-    channel and whether the channel is blacklisted.
+    channel.
 
     A LiquidityHint is the value of a dict, which is keyed to node ids and the
     channel.
@@ -181,7 +188,6 @@ class LiquidityHint:
         self._cannot_send_forward = None
         self._can_send_backward = None
         self._cannot_send_backward = None
-        self.blacklist_timestamp = 0
         self.hint_timestamp = 0
         self._inflight_htlcs_forward = 0
         self._inflight_htlcs_backward = 0
@@ -294,10 +300,8 @@ class LiquidityHint:
             self._inflight_htlcs_backward = max(0, self._inflight_htlcs_forward - 1)
 
     def __repr__(self):
-        is_blacklisted = False if not self.blacklist_timestamp else int(time.time()) - self.blacklist_timestamp < BLACKLIST_DURATION
         return f"forward: can send: {self._can_send_forward} msat, cannot send: {self._cannot_send_forward} msat, htlcs: {self._inflight_htlcs_forward}\n" \
-               f"backward: can send: {self._can_send_backward} msat, cannot send: {self._cannot_send_backward} msat, htlcs: {self._inflight_htlcs_backward}\n" \
-               f"blacklisted: {is_blacklisted}"
+               f"backward: can send: {self._can_send_backward} msat, cannot send: {self._cannot_send_backward} msat, htlcs: {self._inflight_htlcs_backward}\n"
 
 
 class LiquidityHintMgr:
@@ -379,22 +383,6 @@ class LiquidityHintMgr:
         inflight_htlc_fee = num_inflight_htlcs * success_fee
         return success_fee + inflight_htlc_fee
 
-    @with_lock
-    def add_to_blacklist(self, channel_id: ShortChannelID):
-        hint = self.get_hint(channel_id)
-        now = int(time.time())
-        hint.blacklist_timestamp = now
-
-    @with_lock
-    def get_blacklist(self) -> Set[ShortChannelID]:
-        now = int(time.time())
-        return set(k for k, v in self._liquidity_hints.items() if now - v.blacklist_timestamp < BLACKLIST_DURATION)
-
-    @with_lock
-    def clear_blacklist(self):
-        for k, v in self._liquidity_hints.items():
-            v.blacklist_timestamp = 0
-
     @with_lock
     def reset_liquidity_hints(self):
         for k, v in self._liquidity_hints.items():
@@ -414,6 +402,34 @@ class LNPathFinder(Logger):
         Logger.__init__(self)
         self.channel_db = channel_db
         self.liquidity_hints = LiquidityHintMgr()
+        self._edge_blacklist = dict()  # type: Dict[ShortChannelID, int]  # scid -> expiration
+        self._blacklist_lock = threading.Lock()
+
+    def _is_edge_blacklisted(self, short_channel_id: ShortChannelID, *, now: int) -> bool:
+        blacklist_expiration = self._edge_blacklist.get(short_channel_id)
+        if blacklist_expiration is None:
+            return False
+        if blacklist_expiration < now:
+            return False
+            # TODO rm expired entries from cache (note: perf vs thread-safety)
+        return True
+
+    def add_edge_to_blacklist(
+        self,
+        short_channel_id: ShortChannelID,
+        *,
+        now: int = None,
+        duration: int = 3600,  # seconds
+    ) -> None:
+        if now is None:
+            now = int(time.time())
+        with self._blacklist_lock:
+            blacklist_expiration = self._edge_blacklist.get(short_channel_id, 0)
+            self._edge_blacklist[short_channel_id] = max(blacklist_expiration, now + duration)
+
+    def clear_blacklist(self):
+        with self._blacklist_lock:
+            self._edge_blacklist = dict()
 
     def update_liquidity_hints(
             self,
@@ -447,7 +463,7 @@ class LNPathFinder(Logger):
     def _edge_cost(
             self,
             *,
-            short_channel_id: bytes,
+            short_channel_id: ShortChannelID,
             start_node: bytes,
             end_node: bytes,
             payment_amt_msat: int,
@@ -455,10 +471,13 @@ class LNPathFinder(Logger):
             is_mine=False,
             my_channels: Dict[ShortChannelID, 'Channel'] = None,
             private_route_edges: Dict[ShortChannelID, RouteEdge] = None,
+            now: int,  # unix ts
     ) -> Tuple[float, int]:
         """Heuristic cost (distance metric) of going through a channel.
         Returns (heuristic_cost, fee_for_edge_msat).
         """
+        if self._is_edge_blacklisted(short_channel_id, now=now):
+            return float('inf'), 0
         if private_route_edges is None:
             private_route_edges = {}
         channel_info = self.channel_db.get_channel_info(
@@ -466,12 +485,12 @@ class LNPathFinder(Logger):
         if channel_info is None:
             return float('inf'), 0
         channel_policy = self.channel_db.get_policy_for_node(
-            short_channel_id, start_node, my_channels=my_channels, private_route_edges=private_route_edges)
+            short_channel_id, start_node, my_channels=my_channels, private_route_edges=private_route_edges, now=now)
         if channel_policy is None:
             return float('inf'), 0
         # channels that did not publish both policies often return temporary channel failure
         channel_policy_backwards = self.channel_db.get_policy_for_node(
-            short_channel_id, end_node, my_channels=my_channels, private_route_edges=private_route_edges)
+            short_channel_id, end_node, my_channels=my_channels, private_route_edges=private_route_edges, now=now)
         if (channel_policy_backwards is None
                 and not is_mine
                 and short_channel_id not in private_route_edges):
@@ -489,6 +508,12 @@ class LNPathFinder(Logger):
         route_edge = private_route_edges.get(short_channel_id, None)
         if route_edge is None:
             node_info = self.channel_db.get_node_info_for_node_id(node_id=end_node)
+            if node_info:
+                # it's ok if we are missing the node_announcement (node_info) for this node,
+                # but if we have it, we enforce that they support var_onion_optin
+                node_features = LnFeatures(node_info.features)
+                if not node_features.supports(LnFeatures.VAR_ONION_OPT):  # note: this is kind of slow. could be cached.
+                    return float('inf'), 0
             route_edge = RouteEdge.from_channel_policy(
                 channel_policy=channel_policy,
                 short_channel_id=short_channel_id,
@@ -506,7 +531,7 @@ class LNPathFinder(Logger):
         if ignore_costs:
             return DEFAULT_PENALTY_BASE_MSAT, 0
         fee_msat = route_edge.fee_for_edge(payment_amt_msat)
-        cltv_cost = route_edge.cltv_expiry_delta * payment_amt_msat * 15 / 1_000_000_000
+        cltv_cost = route_edge.cltv_delta * payment_amt_msat * 15 / 1_000_000_000
         # the liquidty penalty takes care we favor edges that should be able to forward
         # the payment and penalize edges that cannot
         liquidity_penalty = self.liquidity_hints.penalty(start_node, end_node, short_channel_id, payment_amt_msat)
@@ -528,12 +553,12 @@ class LNPathFinder(Logger):
         # run Dijkstra
         # The search is run in the REVERSE direction, from nodeB to nodeA,
         # to properly calculate compound routing fees.
-        blacklist = self.liquidity_hints.get_blacklist()
         distance_from_start = defaultdict(lambda: float('inf'))
         distance_from_start[nodeB] = 0
         previous_hops = {}  # type: Dict[bytes, PathEdge]
         nodes_to_explore = queue.PriorityQueue()
         nodes_to_explore.put((0, invoice_amount_msat, nodeB))  # order of fields (in tuple) matters!
+        now = int(time.time())
 
         # main loop of search
         while nodes_to_explore.qsize() > 0:
@@ -560,7 +585,7 @@ class LNPathFinder(Logger):
 
             for edge_channel_id in channels_for_endnode:
                 assert isinstance(edge_channel_id, bytes)
-                if blacklist and edge_channel_id in blacklist:
+                if self._is_edge_blacklisted(edge_channel_id, now=now):
                     continue
                 channel_info = self.channel_db.get_channel_info(
                     edge_channel_id, my_channels=my_sending_channels, private_route_edges=private_route_edges)
@@ -580,7 +605,9 @@ class LNPathFinder(Logger):
                     ignore_costs=(edge_startnode == nodeA),
                     is_mine=is_mine,
                     my_channels=my_sending_channels,
-                    private_route_edges=private_route_edges)
+                    private_route_edges=private_route_edges,
+                    now=now,
+                )
                 alt_dist_to_neighbour = distance_from_start[edge_endnode] + edge_cost
                 if alt_dist_to_neighbour < distance_from_start[edge_startnode]:
                     distance_from_start[edge_startnode] = alt_dist_to_neighbour
