@@ -40,7 +40,7 @@ import functools
 from enum import IntEnum
 
 import aiorpcx
-from aiorpcx import ignore_after
+from aiorpcx import ignore_after, NetAddress
 from aiohttp import ClientResponse
 
 from . import util
@@ -48,7 +48,7 @@ from .util import (log_exceptions, ignore_exceptions, OldTaskGroup,
                    bfh, make_aiohttp_session, send_exception_to_crash_reporter,
                    is_hash256_str, is_non_negative_integer, MyEncoder, NetworkRetryManager,
                    nullcontext, error_text_str_to_safe_str)
-from .bitcoin import COIN
+from .bitcoin import COIN, DummyAddress, DummyAddressUsedInTxException
 from . import constants
 from . import blockchain
 from . import bitcoin
@@ -59,7 +59,6 @@ from .interface import (Interface, PREFERRED_NETWORK_PROTOCOL,
                         RequestTimedOut, NetworkTimeout, BUCKET_NAME_OF_ONION_SERVERS,
                         NetworkException, RequestCorrupted, ServerAddr)
 from .version import PROTOCOL_VERSION
-from .simple_config import SimpleConfig
 from .i18n import _
 from .logging import get_logger, Logger
 
@@ -71,6 +70,7 @@ if TYPE_CHECKING:
     from .lnworker import LNGossip
     from .lnwatcher import WatchTower
     from .daemon import Daemon
+    from .simple_config import SimpleConfig
 
 
 _logger = get_logger(__name__)
@@ -168,35 +168,51 @@ proxy_modes = ['socks4', 'socks5']
 def serialize_proxy(p):
     if not isinstance(p, dict):
         return None
-    return ':'.join([p.get('mode'), p.get('host'), p.get('port'),
-                     p.get('user', ''), p.get('password', '')])
+    return ':'.join([p.get('mode'), p.get('host'), p.get('port')])
 
 
-def deserialize_proxy(s: str) -> Optional[dict]:
+def deserialize_proxy(s: Optional[str], user: str = None, password: str = None) -> Optional[dict]:
     if not isinstance(s, str):
         return None
     if s.lower() == 'none':
         return None
-    proxy = {"mode":"socks5", "host":"localhost"}
-    # FIXME raw IPv6 address fails here
+    proxy = {"mode": "socks5", "host": "localhost"}
+
     args = s.split(':')
-    n = 0
-    if proxy_modes.count(args[n]) == 1:
-        proxy["mode"] = args[n]
-        n += 1
-    if len(args) > n:
-        proxy["host"] = args[n]
-        n += 1
-    if len(args) > n:
-        proxy["port"] = args[n]
-        n += 1
-    else:
-        proxy["port"] = "8080" if proxy["mode"] == "http" else "1080"
-    if len(args) > n:
-        proxy["user"] = args[n]
-        n += 1
-    if len(args) > n:
-        proxy["password"] = args[n]
+    if args[0] in proxy_modes:
+        proxy['mode'] = args[0]
+        args = args[1:]
+
+    def is_valid_port(ps: str):
+        try:
+            return 0 < int(ps) < 65535
+        except ValueError:
+            return False
+
+    def is_valid_host(ph: str):
+        try:
+            NetAddress(ph, '1')
+        except ValueError:
+            return False
+        return True
+
+    # detect migrate from old settings
+    if len(args) == 4 and is_valid_host(args[0]) and is_valid_port(args[1]):  # host:port:user:pass,
+        proxy['host'] = args[0]
+        proxy['port'] = args[1]
+        proxy['user'] = args[2]
+        proxy['password'] = args[3]
+        return proxy
+
+    proxy['host'] = ':'.join(args[:-1])
+    proxy['port'] = args[-1]
+
+    if not is_valid_host(proxy['host']) or not is_valid_port(proxy['port']):
+        return None
+
+    proxy['user'] = user
+    proxy['password'] = password
+
     return proxy
 
 
@@ -238,14 +254,21 @@ class UntrustedServerReturnedError(NetworkException):
     def get_message_for_gui(self) -> str:
         return str(self)
 
-    def __str__(self):
-        return _("The server returned an error.")
-
-    def __repr__(self):
+    def get_untrusted_message(self) -> str:
         e = self.original_exception
         return (f"<UntrustedServerReturnedError "
                 f"[DO NOT TRUST THIS MESSAGE] original_exception: {error_text_str_to_safe_str(repr(e))}>")
 
+    def __str__(self):
+        # We should not show the untrusted text from self.original_exception,
+        # to avoid accidentally showing it in the GUI.
+        return _("The server returned an error.")
+
+    def __repr__(self):
+        # We should not show the untrusted text from self.original_exception,
+        # to avoid accidentally showing it in the GUI.
+        return f"<UntrustedServerReturnedError {str(self)!r}>"
+
 
 _INSTANCE = None
 
@@ -270,7 +293,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
     local_watchtower: Optional['WatchTower'] = None
     path_finder: Optional['LNPathFinder'] = None
 
-    def __init__(self, config: SimpleConfig, *, daemon: 'Daemon' = None):
+    def __init__(self, config: 'SimpleConfig', *, daemon: 'Daemon' = None):
         global _INSTANCE
         assert _INSTANCE is None, "Network is a singleton!"
         _INSTANCE = self
@@ -287,21 +310,21 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         self.asyncio_loop = util.get_asyncio_loop()
         assert self.asyncio_loop.is_running(), "event loop not running"
 
-        assert isinstance(config, SimpleConfig), f"config should be a SimpleConfig instead of {type(config)}"
         self.config = config
-
         self.daemon = daemon
 
         blockchain.read_blockchains(self.config)
         blockchain.init_headers_file_for_best_chain()
         self.logger.info(f"blockchains {list(map(lambda b: b.forkpoint, blockchain.blockchains.values()))}")
-        self._blockchain_preferred_block = self.config.get('blockchain_preferred_block', None)  # type: Dict[str, Any]
+        self._blockchain_preferred_block = self.config.BLOCKCHAIN_PREFERRED_BLOCK  # type: Dict[str, Any]
         if self._blockchain_preferred_block is None:
             self._set_preferred_chain(None)
         self._blockchain = blockchain.get_best_chain()
 
         self._allowed_protocols = {PREFERRED_NETWORK_PROTOCOL}
 
+        self.proxy = None
+        self.is_proxy_tor = None
         self._init_parameters_from_config()
 
         self.taskgroup = None
@@ -342,7 +365,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         self._was_started = False
 
         # lightning network
-        if self.config.get('run_watchtower', False):
+        if self.config.WATCHTOWER_SERVER_ENABLED:
             from . import lnwatcher
             self.local_watchtower = lnwatcher.WatchTower(self)
             asyncio.ensure_future(self.local_watchtower.start_watching())
@@ -358,13 +381,13 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         from . import lnrouter
         from . import channel_db
         from . import lnworker
-        if not self.config.get('use_gossip'):
+        if not self.config.LIGHTNING_USE_GOSSIP:
             return
         if self.lngossip is None:
             self.channel_db = channel_db.ChannelDB(self)
             self.path_finder = lnrouter.LNPathFinder(self.channel_db)
             self.channel_db.load_data()
-            self.lngossip = lnworker.LNGossip()
+            self.lngossip = lnworker.LNGossip(self.config)
             self.lngossip.start_network(self)
 
     async def stop_gossip(self, *, full_shutdown: bool = False):
@@ -386,6 +409,9 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
 
     @staticmethod
     def get_instance() -> Optional["Network"]:
+        """Return the global singleton network instance.
+        Note that this can return None! If we are run with the --offline flag, there is no network.
+        """
         return _INSTANCE
 
     def with_recent_servers_lock(func):
@@ -479,7 +505,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         self.config.requested_fee_estimates()
         histogram = await interface.get_fee_histogram()
         self.config.mempool_fees = histogram
-        self.logger.info(f'fee_histogram {histogram}')
+        self.logger.info(f'fee_histogram {len(histogram)}')
         util.trigger_callback('fee_histogram', self.config.mempool_fees)
 
     def get_parameters(self) -> NetworkParameters:
@@ -489,9 +515,11 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
                                  oneserver=self.oneserver)
 
     def _init_parameters_from_config(self) -> None:
-        self.auto_connect = self.config.get('auto_connect', True)
+        dns_hacks.configure_dns_resolver()
+        self.auto_connect = self.config.NETWORK_AUTO_CONNECT
         self._set_default_server()
-        self._set_proxy(deserialize_proxy(self.config.get('proxy')))
+        self._set_proxy(deserialize_proxy(self.config.NETWORK_PROXY, self.config.NETWORK_PROXY_USER,
+                                          self.config.NETWORK_PROXY_PASSWORD))
         self._maybe_set_oneserver()
 
     def get_donation_address(self):
@@ -554,7 +582,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
             else:
                 out[server.host] = {server.protocol: port}
         # potentially filter out some
-        if self.config.get('noonion'):
+        if self.config.NETWORK_NOONION:
             out = filter_noonion(out)
         return out
 
@@ -590,7 +618,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
 
     def _set_default_server(self) -> None:
         # Server for addresses and transactions
-        server = self.config.get('server', None)
+        server = self.config.NETWORK_SERVER
         # Sanitize default server
         if server:
             try:
@@ -603,23 +631,36 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         assert isinstance(self.default_server, ServerAddr), f"invalid type for default_server: {self.default_server!r}"
 
     def _set_proxy(self, proxy: Optional[dict]):
-        self.proxy = proxy
-        dns_hacks.configure_dns_depending_on_proxy(bool(proxy))
+        if self.proxy == proxy:
+            return
+
         self.logger.info(f'setting proxy {proxy}')
+        self.proxy = proxy
+        self.is_proxy_tor = None
 
-        self.tor_proxy = False
-        if bool(proxy) and proxy['mode'] == 'socks5':
-            # test for Tor
-            self.tor_proxy = util.is_tor_socks_port(proxy['host'], int(proxy['port']))
-            if self.tor_proxy:
-                self.logger.info(f'Proxy is TOR')
+        def tor_probe_task(p):
+            assert p is not None
+            tor_proxy = util.is_tor_socks_port(p['host'], int(p['port']))
+            if self.proxy == p:  # is this the proxy we probed?
+                self.logger.info(f'Proxy is {"" if tor_proxy else "not "}TOR')
+                self._tor_probe_done(tor_proxy)
 
-        util.trigger_callback('proxy_set', self.proxy, self.tor_proxy)
+        if proxy and proxy['mode'] == 'socks5':
+            t = threading.Thread(target=tor_probe_task, args=(proxy,), daemon=True)
+            t.start()
+
+        util.trigger_callback('proxy_set', self.proxy)
+
+    def _tor_probe_done(self, is_tor: bool):
+        self.is_proxy_tor = is_tor
+        util.trigger_callback('tor_probed', is_tor)
 
     @log_exceptions
     async def set_parameters(self, net_params: NetworkParameters):
         proxy = net_params.proxy
         proxy_str = serialize_proxy(proxy)
+        proxy_user = proxy['user'] if proxy else None
+        proxy_pass = proxy['password'] if proxy else None
         server = net_params.server
         # sanitize parameters
         try:
@@ -628,14 +669,18 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
                 int(proxy['port'])
         except Exception:
             return
-        self.config.set_key('auto_connect', net_params.auto_connect, False)
-        self.config.set_key('oneserver', net_params.oneserver, False)
-        self.config.set_key('proxy', proxy_str, False)
-        self.config.set_key('server', str(server), True)
+        self.config.NETWORK_AUTO_CONNECT = net_params.auto_connect
+        self.config.NETWORK_ONESERVER = net_params.oneserver
+        self.config.NETWORK_PROXY = proxy_str
+        self.config.NETWORK_PROXY_USER = proxy_user
+        self.config.NETWORK_PROXY_PASSWORD = proxy_pass
+        self.config.NETWORK_SERVER = str(server)
         # abort if changes were not allowed by config
-        if self.config.get('server') != str(server) \
-                or self.config.get('proxy') != proxy_str \
-                or self.config.get('oneserver') != net_params.oneserver:
+        if self.config.NETWORK_SERVER != str(server) \
+                or self.config.NETWORK_PROXY != proxy_str \
+                or self.config.NETWORK_PROXY_USER != proxy_user \
+                or self.config.NETWORK_PROXY_PASSWORD != proxy_pass \
+                or self.config.NETWORK_ONESERVER != net_params.oneserver:
             return
 
         proxy_changed = self.proxy != proxy
@@ -657,7 +702,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         util.trigger_callback('network_updated')
 
     def _maybe_set_oneserver(self) -> None:
-        oneserver = bool(self.config.get('oneserver', False))
+        oneserver = self.config.NETWORK_ONESERVER
         self.oneserver = oneserver
         self.num_server = NUM_TARGET_CONNECTED_SERVERS if not oneserver else 0
 
@@ -788,8 +833,8 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         util.trigger_callback('network_updated')
 
     def get_network_timeout_seconds(self, request_type=NetworkTimeout.Generic) -> int:
-        if self.config.get('network_timeout', None):
-            return int(self.config.get('network_timeout'))
+        if self.config.NETWORK_TIMEOUT:
+            return self.config.NETWORK_TIMEOUT
         if self.oneserver and not self.auto_connect:
             return request_type.MOST_RELAXED
         if self.proxy:
@@ -813,7 +858,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         # note: using longer timeouts here as DNS can sometimes be slow!
         timeout = self.get_network_timeout_seconds(NetworkTimeout.Generic)
         try:
-            await asyncio.wait_for(interface.ready, timeout)
+            await util.wait_for2(interface.ready, timeout)
         except BaseException as e:
             self.logger.info(f"couldn't launch iface {server} -- {repr(e)}")
             await interface.close()
@@ -901,7 +946,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
             except aiorpcx.jsonrpc.CodeMessageError as e:
                 wrapped_exc = UntrustedServerReturnedError(original_exception=e)
                 # log (sanitized) untrusted error text now, to ease debugging
-                self.logger.debug(f"got error from server for {func.__qualname__}: {wrapped_exc!r}")
+                self.logger.debug(f"got error from server for {func.__qualname__}: {wrapped_exc.get_untrusted_message()!r}")
                 raise wrapped_exc from e
         return wrapper
 
@@ -918,6 +963,8 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
             raise RequestTimedOut()
         if timeout is None:
             timeout = self.get_network_timeout_seconds(NetworkTimeout.Urgent)
+        if any(DummyAddress.is_dummy_address(txout.address) for txout in tx.outputs()):
+            raise DummyAddressUsedInTxException("tried to broadcast tx with dummy address!")
         try:
             out = await self.interface.session.send_request('blockchain.transaction.broadcast', [tx.serialize()], timeout=timeout)
             # note: both 'out' and exception messages are untrusted input from the server
@@ -1191,7 +1238,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
             'height': height,
             'hash': header_hash,
         }
-        self.config.set_key('blockchain_preferred_block', self._blockchain_preferred_block)
+        self.config.BLOCKCHAIN_PREFERRED_BLOCK = self._blockchain_preferred_block
 
     async def follow_chain_given_id(self, chain_id: str) -> None:
         bc = blockchain.blockchains.get(chain_id)
@@ -1403,7 +1450,7 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
         async def get_response(server: ServerAddr):
             interface = Interface(network=self, server=server, proxy=self.proxy)
             try:
-                await asyncio.wait_for(interface.ready, timeout)
+                await util.wait_for2(interface.ready, timeout)
             except BaseException as e:
                 await interface.close()
                 return
