@@ -1,7 +1,7 @@
 import os
 import base64
 import json
-from typing import Optional
+from typing import Optional, TYPE_CHECKING
 
 from electrum import bip32, constants
 from electrum.crypto import sha256
@@ -10,14 +10,16 @@ from electrum.keystore import Hardware_KeyStore
 from electrum.transaction import Transaction
 from electrum.wallet import Multisig_Wallet
 from electrum.util import UserFacingException
-from electrum.base_wizard import ScriptTypeNotSupported
 from electrum.logging import get_logger
 from electrum.plugin import runs_in_hwd_thread, Device
 from electrum.network import Network
 
-from ..hw_wallet import HW_PluginBase, HardwareClientBase
-from ..hw_wallet.plugin import OutdatedHwFirmwareException
+from electrum.plugins.hw_wallet import HW_PluginBase, HardwareClientBase
+from electrum.plugins.hw_wallet.plugin import OutdatedHwFirmwareException
 
+if TYPE_CHECKING:
+    from electrum.plugin import DeviceInfo
+    from electrum.wizard import NewWalletWizard
 
 _logger = get_logger(__name__)
 
@@ -436,22 +438,6 @@ class JadePlugin(HW_PluginBase):
 
         return client
 
-    def setup_device(self, device_info, wizard, purpose):
-        device_id = device_info.device.id_
-        client = self.scan_and_create_client_for_device(device_id=device_id, wizard=wizard)
-
-        # Call authenticate on hww to ensure unlocked and suitable for network
-        # May involve user entering PIN on (or even setting up!) hardware device
-        wizard.run_task_without_blocking_gui(task=lambda: client.authenticate())
-        return client
-
-    def get_xpub(self, device_id, derivation, xtype, wizard):
-        if xtype not in self.SUPPORTED_XTYPES:
-            raise ScriptTypeNotSupported(_('This type of script is not supported with {}.').format(self.device))
-        client = self.scan_and_create_client_for_device(device_id=device_id, wizard=wizard)
-        xpub = client.get_xpub(derivation, xtype)
-        return xpub
-
     def show_address(self, wallet, address, keystore=None):
         if keystore is None:
             keystore = wallet.get_keystore()
@@ -476,3 +462,27 @@ class JadePlugin(HW_PluginBase):
 
         if hw_address != address:
             keystore.handler.show_error(_('The address generated by {} does not match!').format(self.device))
+
+    def wizard_entry_for_device(self, device_info: 'DeviceInfo', *, new_wallet=True) -> str:
+        if new_wallet:
+            return 'jade_start' if device_info.initialized else 'jade_not_initialized'
+        else:
+            return 'jade_unlock'
+
+    # insert jade pages in new wallet wizard
+    def extend_wizard(self, wizard: 'NewWalletWizard'):
+        views = {
+            'jade_start': {
+                'next': 'jade_xpub',
+            },
+            'jade_xpub': {
+                'next': lambda d: wizard.wallet_password_view(d) if wizard.last_cosigner(d) else 'multisig_cosigner_keystore',
+                'accept': wizard.maybe_master_pubkey,
+                'last': lambda d: wizard.is_single_password() and wizard.last_cosigner(d)
+            },
+            'jade_not_initialized': {},
+            'jade_unlock': {
+                'last': True
+            },
+        }
+        wizard.navmap_merge(views)
